@@ -1,4 +1,4 @@
-/*      $Id: lircd.c,v 5.57 2005/07/10 08:34:12 lirc Exp $      */
+/*      $Id: lircd.c,v 5.84 2009/04/28 16:25:50 lirc Exp $      */
 
 /****************************************************************************
  ** lircd.c *****************************************************************
@@ -38,6 +38,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <ctype.h>
 #include <string.h>
 #include <signal.h>
 #include <unistd.h>
@@ -56,6 +57,16 @@
 #include <fcntl.h>
 #include <sys/file.h>
 
+#if defined(__linux__)
+#include <linux/input.h>
+#include <linux/uinput.h>
+#include "input_map.h"
+#endif
+
+#if defined __APPLE__
+#include <sys/ioccom.h>
+#endif
+
 #ifndef timersub
 #define timersub(a, b, result)                                            \
   do {                                                                    \
@@ -73,6 +84,7 @@
 #include "config_file.h"
 #include "hardware.h"
 #include "hw-types.h"
+#include "release.h"
 
 struct ir_remote *remotes;
 struct ir_remote *free_remotes=NULL;
@@ -87,7 +99,7 @@ static char *repeat_message=NULL;
 
 extern struct hardware hw;
 
-char *progname="lircd " VERSION;
+char *progname="lircd";
 char *configfile=LIRCDCFGFILE;
 #ifndef USE_SYSLOG
 char *logfile=LOGFILE;
@@ -106,6 +118,7 @@ struct protocol_directive directives[] =
 	{"SEND_STOP",send_stop},
 	{"VERSION",version},
 	{"SET_TRANSMITTERS",set_transmitters},
+	{"SIMULATE",simulate},
 	{NULL,NULL}
 	/*
 	{"DEBUG",debug},
@@ -132,6 +145,9 @@ char *protocol_string[] =
 	"SIGHUP\n"
 };
 
+static void log_enable(int enabled);
+static int log_enabled = 1;
+
 #ifndef USE_SYSLOG
 #define HOSTNAME_LEN 128
 char hostname[HOSTNAME_LEN+1];
@@ -139,30 +155,42 @@ char hostname[HOSTNAME_LEN+1];
 FILE *lf=NULL;
 #endif
 
-/* fixme: */
+/* quite arbitrary limits */
 #define MAX_PEERS	100
+/* substract one for lirc, sockfd, sockinet, logfile, pidfile, uinput */
+#define MAX_CLIENTS     (FD_SETSIZE-6-MAX_PEERS)
 
 int sockfd, sockinet;
-int clis[FD_SETSIZE-5-MAX_PEERS]; /* substract one for lirc, sockfd, sockinet, logfile, pidfile */
+static int uinputfd = -1;
+int clis[MAX_CLIENTS];
 
 #define CT_LOCAL  1
 #define CT_REMOTE 2
 
-int cli_type[FD_SETSIZE-5-MAX_PEERS];
-int clin=0;
+static int cli_type[MAX_CLIENTS];
+static int clin=0;
 
 int listen_tcpip=0;
 unsigned short int port=LIRC_INET_PORT;
+struct in_addr address;
 
 struct	peer_connection *peers[MAX_PEERS];
 int	peern = 0;
 
 int debug=0;
-int daemonized=0;
+static int daemonized=0;
+static int allow_simulate=0;
+static int userelease=0;
+static int useuinput=0;
 
 static sig_atomic_t term=0,hup=0,alrm=0;
 static int termsig;
 
+inline int use_hw()
+{
+	return (clin>0 || (useuinput && uinputfd != -1) || repeat_remote != NULL);
+}
+
 /* set_transmitters only supports 32 bit int */
 #define MAX_TX (CHAR_BIT*sizeof(unsigned long))
 
@@ -196,13 +224,20 @@ oatoi (s)
 /* A safer write(), since sockets might not write all but only some of the
    bytes requested */
 
-inline int write_socket(int fd, char *buf, int len)
+inline int write_socket(int fd, const char *buf, int len)
 {
 	int done,todo=len;
 
 	while(todo)
 	{
+#ifdef SIM_REC
+		do{
+			done=write(fd,buf,todo);
+		}
+		while(done<0 && errno == EAGAIN);
+#else
 		done=write(fd,buf,todo);
+#endif
 		if(done<=0) return(done);
 		buf+=done;
 		todo-=done;
@@ -210,7 +245,7 @@ inline int write_socket(int fd, char *buf, int len)
 	return(len);
 }
 
-inline int write_socket_len(int fd, char *buf)
+inline int write_socket_len(int fd, const char *buf)
 {
 	int len;
 
@@ -283,6 +318,7 @@ void dosigterm(int sig)
 		free_config(free_remotes);
 	}
 	free_config(remotes);
+	repeat_remote = NULL;
 	logprintf(LOG_NOTICE,"caught signal");
 	for (i=0; i<clin; i++)
 	{
@@ -291,6 +327,13 @@ void dosigterm(int sig)
 	};
 	shutdown(sockfd,2);
 	close(sockfd);
+
+	if(uinputfd != -1)
+	{
+		ioctl(uinputfd, UI_DEV_DESTROY);
+		close(uinputfd);
+		uinputfd = -1;
+	}
 	if(listen_tcpip)
 	{
 		shutdown(sockinet,2);
@@ -298,7 +341,7 @@ void dosigterm(int sig)
 	}
 	fclose(pidf);
 	(void) unlink(pidfile);
-	if(clin>0 && hw.deinit_func) hw.deinit_func();
+	if(use_hw() && hw.deinit_func) hw.deinit_func();
 #ifdef USE_SYSLOG
 	closelog();
 #else
@@ -340,7 +383,7 @@ void dosighup(int sig)
 	if(-1==fchmod(fileno(lf),s.st_mode))
 	{
 		logprintf(LOG_WARNING,"could not set file permissions");
-		logperror(0,NULL);
+		logperror(LOG_WARNING,NULL);
 	}
 #endif
 
@@ -367,6 +410,61 @@ void dosighup(int sig)
       }
 }
 
+int setup_uinputfd(const char *name)
+{
+#if defined(__linux__)
+	int fd;
+	int key;
+	struct uinput_user_dev dev;
+	
+	fd = open("/dev/input/uinput", O_RDWR);
+	if(fd == -1)
+	{
+		fd = open("/dev/uinput", O_RDWR);
+		if(fd == -1)
+		{
+			fd = open("/dev/misc/uinput", O_RDWR);
+			if(fd == -1)
+			{
+				fprintf(stderr, "could not open %s\n",
+					"uinput");
+				perror(NULL);
+				return -1;
+			}
+		}
+	}
+	memset(&dev, 0, sizeof(dev));
+	strncpy(dev.name, name, sizeof(dev.name));
+	dev.name[sizeof(dev.name)-1] = 0;
+	if(write(fd, &dev, sizeof(dev)) != sizeof(dev) ||
+	   ioctl(fd, UI_SET_EVBIT, EV_KEY) != 0 ||
+	   ioctl(fd, UI_SET_EVBIT, EV_REP) != 0)
+	{
+		goto setup_error;
+	}
+
+        for(key = KEY_RESERVED; key <= KEY_UNKNOWN; key++)
+	{
+                if(ioctl(fd, UI_SET_KEYBIT, key) != 0)
+		{
+			goto setup_error;
+                }
+	}
+
+	if(ioctl(fd, UI_DEV_CREATE) != 0)
+	{
+		goto setup_error;
+	}
+	return fd;
+	
+ setup_error:
+	fprintf(stderr, "could not setup %s\n", "uinput");
+	perror(NULL);
+	close(fd);
+#endif
+	return -1;
+}
+
 void config(void)
 {
 	FILE *fd;
@@ -386,7 +484,7 @@ void config(void)
 		logperror(LOG_ERR,NULL);
 		return;
 	}
-	config_remotes=read_config(fd);
+	config_remotes=read_config(fd, configfile);
 	fclose(fd);
 	if(config_remotes==(void *) -1)
 	{
@@ -428,9 +526,7 @@ void remove_client(int fd)
 			logprintf(LOG_INFO,"removed client");
 			
 			clin--;
-			if(clin==0 &&
-			   repeat_remote==NULL &&
-			   hw.deinit_func)
+			if(!use_hw() && hw.deinit_func)
 			{
 				hw.deinit_func();
 			}
@@ -447,7 +543,7 @@ void remove_client(int fd)
 void add_client(int sock)
 {
 	int fd;
-	int clilen;
+	socklen_t clilen;
 	struct sockaddr client_addr;
 	int flags;
 
@@ -460,7 +556,7 @@ void add_client(int sock)
 		dosigterm(SIGTERM);
 	};
 
-	if(fd>=FD_SETSIZE)
+	if(fd>=FD_SETSIZE || clin>=MAX_CLIENTS)
 	{
 		logprintf(LOG_ERR,"connection rejected");
 		shutdown(fd,2);
@@ -488,20 +584,24 @@ void add_client(int sock)
 	{
 		cli_type[clin]=0; /* what? */
 	}
-	clis[clin++]=fd;
-	if(clin==1 && repeat_remote==NULL)
+	clis[clin]=fd;
+	if(!use_hw())
 	{
 		if(hw.init_func)
 		{
 			if(!hw.init_func())
 			{
-				shutdown(clis[0],2);
-				close(clis[0]);
-				clin=0;
-				dosigterm(SIGTERM);
+				logprintf(LOG_WARNING,
+					  "Failed to initialize hardware");
+				/* Don't exit here, otherwise lirc
+				 * bails out, and lircd exits, making
+				 * it impossible to connect to when we
+				 * have a device actually plugged
+				 * in. */
 			}
 		}
 	}
+	clin++;
 }
 
 int add_peer_connection(char *server)
@@ -642,8 +742,8 @@ int get_peer_message(struct peer_connection *peer)
 	if(length)
 	{
 		buffer[length]=0;
-		end=strchr(buffer,'\n');
-		if(end==NULL)
+		end=strrchr(buffer,'\n');
+		if(end == NULL || end[1] != 0)
 		{
 			logprintf(LOG_ERR,"bad send packet: \"%s\"",buffer);
 			/* remove clients that behave badly */
@@ -651,6 +751,7 @@ int get_peer_message(struct peer_connection *peer)
 		}
 		end++;	/* include the \n */
 		end[0]=0;
+		length=strlen(buffer);
 		LOGPRINTF(1,"received peer message: \"%s\"",buffer);
 		for(i=0;i<clin;i++)
 		{
@@ -776,6 +877,10 @@ void start_server(mode_t permission,int nodaemon)
 	listen(sockfd,3);
 	nolinger(sockfd);
 
+	if(useuinput)
+	{
+		uinputfd = setup_uinputfd(progname);
+	}
 	if(listen_tcpip)
 	{
 		int enable=1;
@@ -792,7 +897,7 @@ void start_server(mode_t permission,int nodaemon)
 		(void) setsockopt(sockinet,SOL_SOCKET,SO_REUSEADDR,
 				  &enable,sizeof(enable));
 		serv_addr_in.sin_family=AF_INET;
-		serv_addr_in.sin_addr.s_addr=htonl(INADDR_ANY);
+		serv_addr_in.sin_addr=address;
 		serv_addr_in.sin_port=htons(port);
 		
 		if(bind(sockinet,(struct sockaddr *) &serv_addr_in,
@@ -848,36 +953,72 @@ void start_server(mode_t permission,int nodaemon)
 	exit(EXIT_FAILURE);
 }
 
-#ifndef USE_SYSLOG
+void log_enable(int enabled)
+{
+	log_enabled = enabled;
+}
+
+#ifdef USE_SYSLOG
 void logprintf(int prio,char *format_str, ...)
 {
-	time_t current;
-	char *currents;
-	va_list ap;  
+	int save_errno = errno;
+	va_list ap;
 	
-	current=time(&current);
-	currents=ctime(&current);
+	if(!log_enabled) return;
 	
-	if(lf) fprintf(lf,"%15.15s %s %s: ",currents+4,hostname,progname);
-	if(!daemonized) fprintf(stderr,"%s: ",progname);
 	va_start(ap,format_str);
-	if(lf)
-	{
-		if(prio==LOG_WARNING) fprintf(lf,"WARNING: ");
-		vfprintf(lf,format_str,ap);
-		fputc('\n',lf);fflush(lf);
-	}
-	if(!daemonized)
-	{
-		if(prio==LOG_WARNING) fprintf(stderr,"WARNING: ");
-		vfprintf(stderr,format_str,ap);
-		fputc('\n',stderr);fflush(stderr);
-	}
+	vsyslog(prio, format_str, ap);
 	va_end(ap);
+
+	errno = save_errno;
 }
 
 void logperror(int prio,const char *s)
 {
+	if(!log_enabled) return;
+	
+	if((s)!=NULL) syslog(prio,"%s: %m\n",(char *) s);
+	else syslog(prio,"%m\n");
+}
+#else
+void logprintf(int prio,char *format_str, ...)
+{
+	int save_errno = errno;
+	va_list ap;  
+	
+	if(!log_enabled) return;
+	
+	if(lf)
+	{
+		time_t current;
+		char *currents;
+		
+		current=time(&current);
+		currents=ctime(&current);
+		
+		fprintf(lf,"%15.15s %s %s: ",currents+4,hostname,progname);
+		va_start(ap,format_str);
+		if(prio==LOG_WARNING) fprintf(lf,"WARNING: ");
+		vfprintf(lf,format_str,ap);
+		fputc('\n',lf);fflush(lf);
+		va_end(ap);
+	}
+	if(!daemonized)
+	{
+		fprintf(stderr,"%s: ",progname);
+		va_start(ap,format_str);
+		if(prio==LOG_WARNING) fprintf(stderr,"WARNING: ");
+		vfprintf(stderr,format_str,ap);
+		fputc('\n',stderr);fflush(stderr);
+		va_end(ap);
+	}
+	errno = save_errno;
+}
+
+void logperror(int prio,const char *s)
+{
+	if(!log_enabled) return;
+	
 	if(s!=NULL)
 	{
 		logprintf(prio,"%s: %s",s,strerror(errno));
@@ -923,6 +1064,11 @@ void dosigalrm(int sig)
 		/* we received a different code from the original
 		   remote control we could repeat the wrong code so
 		   better stop repeating */
+		if(repeat_fd != -1)
+		{
+			send_error(repeat_fd, repeat_message, "repeating interrupted\n");
+		}
+		
 		repeat_remote=NULL;
 		repeat_code=NULL;
 		repeat_fd=-1;
@@ -931,18 +1077,21 @@ void dosigalrm(int sig)
 			free(repeat_message);
 			repeat_message=NULL;
 		}
-		if(clin==0 && repeat_remote==NULL && hw.deinit_func)
+		if(!use_hw() && hw.deinit_func)
 		{
 			hw.deinit_func();
 		}
 		return;
 	}
-	repeat_remote->repeat_countdown--;
-	if(hw.send_func(repeat_remote,repeat_code) &&
+	if(repeat_code->next == NULL || (repeat_code->transmit_state != NULL && repeat_code->transmit_state->next == NULL))
+	{
+		repeat_remote->repeat_countdown--;
+	}
+	if(send_ir_ncode(repeat_remote,repeat_code) &&
 	   repeat_remote->repeat_countdown>0)
 	{
 		repeat_timer.it_value.tv_sec=0;
-		repeat_timer.it_value.tv_usec=repeat_remote->remaining_gap;
+		repeat_timer.it_value.tv_usec=repeat_remote->min_remaining_gap;
 		repeat_timer.it_interval.tv_sec=0;
 		repeat_timer.it_interval.tv_usec=0;
 		
@@ -958,7 +1107,7 @@ void dosigalrm(int sig)
 		repeat_message=NULL;
 		repeat_fd=-1;
 	}
-	if(clin==0 && repeat_remote==NULL && hw.deinit_func)
+	if(!use_hw() && hw.deinit_func)
 	{
 		hw.deinit_func();
 	}
@@ -983,7 +1132,7 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 	}
 	command=strtok(NULL,WHITE_SPACE);
 	if(command==NULL) return(1);
-	*code=get_ir_code(*remote,command);
+	*code=get_code_by_name(*remote,command);
 	if(*code==NULL)
 	{
 		return(send_error(fd,message,"unknown command: \"%s\"\n",
@@ -1102,7 +1251,7 @@ int send_remote_list(int fd,char *message)
 	while(all)
 	{
 		len=snprintf(buffer,PACKET_SIZE+1,"%s\n",all->name);
-		if(len==PACKET_SIZE+1)
+		if(len>=PACKET_SIZE+1)
 		{
 			len=sprintf(buffer,"name_too_long\n");
 		}
@@ -1155,7 +1304,7 @@ int send_remote(int fd,char *message,struct ir_remote *remote)
 			     codes->code,
 			     codes->name);
 #endif
-		if(len==PACKET_SIZE+1)
+		if(len>=PACKET_SIZE+1)
 		{
 			len=sprintf(buffer,"code_too_long\n");
 		}
@@ -1186,7 +1335,7 @@ int send_name(int fd,char *message,struct ir_ncode *code)
 		     code->code,
 		     code->name);
 #endif
-	if(len==PACKET_SIZE+1)
+	if(len>=PACKET_SIZE+1)
 	{
 		len=sprintf(buffer,"1\ncode_too_long\n");
 	}
@@ -1237,7 +1386,7 @@ int set_transmitters(int fd,char *message,char *arguments)
 	{
 		next_tx_int=-1;
 		next_tx_int = strtoul(next_arg,&end_ptr,10);
-		if(*end_ptr || (next_tx_int == ULONG_MAX && errno == ERANGE))
+		if(*end_ptr || next_tx_int == 0 || (next_tx_int == ULONG_MAX && errno == ERANGE))
 		{
 			return(send_error(fd,message, "invalid argument\n"));
 		}
@@ -1251,7 +1400,7 @@ int set_transmitters(int fd,char *message,char *arguments)
 		channels |= next_tx_hex;
 	}while ((next_arg=strtok(NULL,WHITE_SPACE))!=NULL);
 	
-	retval = hw.ioctl_func(LIRC_SET_TRANSMITTER_MASK, channels);
+	retval = hw.ioctl_func(LIRC_SET_TRANSMITTER_MASK, &channels);
 	if(retval<0)
 	{
 		return(send_error(fd, message, "error - could not set "
@@ -1268,6 +1417,68 @@ string_error:
 	return(send_error(fd,message,"no arguments given\n"));
 }
 
+int simulate(int fd,char *message,char *arguments)
+{
+	int i;
+	char *sim, *s, *space;
+	
+	if(!allow_simulate)
+	{
+		return send_error(fd, message,
+				  "SIMULATE command is disabled\n");
+	}
+	if(arguments==NULL)
+	{
+		return send_error(fd, message, "no arguments given\n");
+	}
+	
+	s=arguments;
+	for(i=0; i<16; i++, s++)
+	{
+		if(!isxdigit(*s)) goto simulate_invalid_event;
+	}
+	if(*s != ' ')
+	{
+		goto simulate_invalid_event;
+	}
+	s++;
+	if(*s == ' ')
+	{
+		goto simulate_invalid_event;
+	}
+	for(; *s != ' '; s++)
+	{
+		if(!isxdigit(*s)) goto simulate_invalid_event;
+	}
+	s++;
+	space = strchr(s, ' ');
+	if(space == NULL || space == s)
+	{
+		goto simulate_invalid_event;
+	}
+	s = space + 1;
+	space = strchr(s, ' ');
+	if(strlen(s) == 0 || space != NULL)
+	{
+		goto simulate_invalid_event;
+	}
+	
+	sim = malloc(strlen(arguments) + 1 + 1);
+	if(sim == NULL)
+	{
+		return send_error(fd, message, "out of memory\n");
+	}
+	strcpy(sim, arguments);
+	strcat(sim, "\n");
+	broadcast_message(sim);
+	free(sim);
+	
+	return(send_success(fd,message));
+ simulate_invalid_event:
+	return send_error(fd, message, "invalid event\n");
+	
+}
+
 int send_once(int fd,char *message,char *arguments)
 {
 	return(send_core(fd,message,arguments,1));
@@ -1310,13 +1521,18 @@ int send_core(int fd,char *message,char *arguments,int once)
 	{
 		remote->toggle_mask_state=0;
 	}
-	if(remote->toggle_bit>0)
-		remote->repeat_state=
-		!remote->repeat_state;
-	if(!hw.send_func(remote,code))
+	if(has_toggle_bit_mask(remote))
+	{
+		remote->toggle_bit_mask_state =
+			(remote->toggle_bit_mask_state^remote->toggle_bit_mask);
+	}
+	code->transmit_state = NULL;
+	if(!send_ir_ncode(remote,code))
 	{
 		return(send_error(fd,message,"transmission failed\n"));
 	}
+	gettimeofday(&remote->last_send,NULL);
+	remote->last_code = code;
 	if(once)
 	{
 		remote->repeat_countdown=max(remote->repeat_countdown,reps);
@@ -1326,13 +1542,13 @@ int send_core(int fd,char *message,char *arguments,int once)
 		/* you've been warned, now we have a limit */
 		remote->repeat_countdown=REPEAT_MAX;
 	}
-	if(remote->repeat_countdown>0)
+	if(remote->repeat_countdown>0 || code->next != NULL)
 	{
 		repeat_remote=remote;
 		repeat_code=code;
 		repeat_timer.it_value.tv_sec=0;
 		repeat_timer.it_value.tv_usec=
-			remote->remaining_gap;
+			remote->min_remaining_gap;
 		repeat_timer.it_interval.tv_sec=0;
 		repeat_timer.it_interval.tv_usec=0;
 		if(once)
@@ -1510,32 +1726,49 @@ void free_old_remotes()
 {
 	struct ir_remote *scan_remotes,*found;
 	struct ir_ncode *code;
-
+	const char *release_event;
+	const char *release_remote_name;
+	const char *release_button_name;
+	
+	if(decoding ==free_remotes) return;
+	
+	release_event = release_map_remotes(free_remotes, remotes,
+					    &release_remote_name,
+					    &release_button_name);
+	if(release_event != NULL)
+	{
+		input_message(release_event,
+			      release_remote_name,
+			      release_button_name,
+			      0,
+			      1);
+	}
 	if(last_remote!=NULL)
 	{
-		scan_remotes=free_remotes;
-		while(scan_remotes!=NULL)
+		if(is_in_remotes(free_remotes, last_remote))
 		{
-			if(last_remote==scan_remotes)
+			logprintf(LOG_INFO, "last_remote found");
+			found=get_ir_remote(remotes,last_remote->name);
+			if(found!=NULL)
 			{
-				found=get_ir_remote(remotes,last_remote->name);
-				if(found!=NULL)
+				code=get_code_by_name(found,last_remote->last_code->name);
+				if(code!=NULL)
 				{
-					code=get_ir_code(found,last_remote->last_code->name);
-					if(code!=NULL)
-					{
-						found->reps=last_remote->reps;
-						found->repeat_state=last_remote->repeat_state;
-						found->remaining_gap=last_remote->remaining_gap;
-						last_remote=found;
-						last_remote->last_code=code;
-					}
+					found->reps=last_remote->reps;
+					found->toggle_bit_mask_state=last_remote->toggle_bit_mask_state;
+					found->min_remaining_gap=last_remote->min_remaining_gap;
+					found->max_remaining_gap=last_remote->max_remaining_gap;
+					found->last_send=last_remote->last_send;
+					last_remote=found;
+					last_remote->last_code=code;
+					logprintf(LOG_INFO, "mapped last_remote");
 				}
-				break;
 			}
-			scan_remotes=scan_remotes->next;
 		}
-		if(scan_remotes==NULL) last_remote=NULL;
+		else
+		{
+			last_remote=NULL;
+		}
 	}
 	/* check if last config is still needed */
 	found=NULL;
@@ -1556,7 +1789,7 @@ void free_old_remotes()
 			found=get_ir_remote(remotes,repeat_remote->name);
 			if(found!=NULL)
 			{
-				code=get_ir_code(found,repeat_code->name);
+				code=get_code_by_name(found,repeat_code->name);
 				if(code!=NULL)
 				{
 					struct itimerval repeat_timer;
@@ -1568,8 +1801,9 @@ void free_old_remotes()
 
 					found->last_code=code;
 					found->last_send=repeat_remote->last_send;
-					found->repeat_state=repeat_remote->repeat_state;
-					found->remaining_gap=repeat_remote->remaining_gap;
+					found->toggle_bit_mask_state=repeat_remote->toggle_bit_mask_state;
+					found->min_remaining_gap=repeat_remote->min_remaining_gap;
+					found->max_remaining_gap=repeat_remote->max_remaining_gap;
 
 					setitimer(ITIMER_REAL,&repeat_timer,&repeat_timer);
 					/* "atomic" (shouldn't be necessary any more) */
@@ -1597,12 +1831,72 @@ void free_old_remotes()
 	}
 }
 
+void input_message(const char *message, const char *remote_name,
+		   const char *button_name, int reps, int release)
+{
+	const char *release_message;
+	const char *release_remote_name;
+	const char *release_button_name;
+
+	release_message = check_release_event(&release_remote_name,
+					      &release_button_name);
+	if(release_message)
+	{
+		input_message(release_message, release_remote_name,
+			      release_button_name, 0, 1);
+	}
+	
+	if(!release || userelease)
+	{
+		broadcast_message(message);
+	}
+	
+#ifdef __linux__
+	if(uinputfd != -1)
+	{
+		linux_input_code input_code;
+		
+		if(reps < 2 &&
+		   get_input_code(button_name, &input_code) != -1)
+		{
+			struct input_event event;
+			
+			memset(&event, 0, sizeof(event));
+			event.type = EV_KEY;
+			event.code = input_code;
+			event.value = release ? 0 : (reps>0 ? 2:1);
+			if(write(uinputfd, &event, sizeof(event)) != sizeof(event))
+			{
+				logprintf(LOG_ERR, "writing to uinput failed");
+				logperror(LOG_ERR, NULL);
+			}
+		}
+	}
+#endif
+}
+
+void broadcast_message(const char *message)
+{
+	int len,i;
+	
+	len=strlen(message);
+	
+	for (i=0; i<clin; i++)
+	{
+		LOGPRINTF(1,"writing to client %d",i);
+		if(write_socket(clis[i],message,len)<len)
+		{
+			remove_client(clis[i]);
+			i--;
+		}			
+	}
+}
 
 int waitfordata(long maxusec)
 {
 	fd_set fds;
 	int maxfd,i,ret,reconnect;
-	struct timeval tv,start,now;
+	struct timeval tv,start,now,timeout,release_time;
 
 	while(1)
 	{
@@ -1632,7 +1926,7 @@ int waitfordata(long maxusec)
 				FD_SET(sockinet,&fds);
 				maxfd=max(maxfd,sockinet);
 			}
-			if(clin>0 && hw.rec_mode!=0)
+			if(use_hw() && hw.rec_mode!=0 && hw.fd!=-1)
 			{
 				FD_SET(hw.fd,&fds);
 				maxfd=max(maxfd,hw.fd);
@@ -1692,10 +1986,42 @@ int waitfordata(long maxusec)
 				tv.tv_sec= maxusec / 1000000;
 				tv.tv_usec=maxusec % 1000000;
 			}
+			if(hw.fd == -1)
+			{
+				/* try to reconnect */
+				timerclear(&timeout);
+				timeout.tv_sec = 1;
+				
+				if(timercmp(&tv, &timeout, >) ||
+				   (!reconnect && !timerisset(&tv)))
+				{
+					tv = timeout;
+				}
+			}
+			get_release_time(&release_time);
+			if(timerisset(&release_time))
+			{
+				gettimeofday(&now,NULL);
+				if(timercmp(&now,&release_time,>))
+				{
+					timerclear(&tv);
+				}
+				else
+				{
+					struct timeval gap;
+					
+					timersub(&release_time, &now, &gap);
+					if(timercmp(&tv, &gap, >))
+					{
+						tv = gap;
+					}
+				}
+			}
 #ifdef SIM_REC
 			ret=select(maxfd+1,&fds,NULL,NULL,NULL);
 #else
-			if(timerisset(&tv) || reconnect)
+			if(timerisset(&tv) || timerisset(&release_time) ||
+			   reconnect)
 			{
 				ret=select(maxfd+1,&fds,NULL,NULL,&tv);
 			}
@@ -1712,6 +2038,25 @@ int waitfordata(long maxusec)
 				continue;
 			}
 			gettimeofday(&now,NULL);
+			if(timerisset(&release_time) &&
+			   timercmp(&now, &release_time, >))
+			{
+				const char *release_message;
+				const char *release_remote_name;
+				const char *release_button_name;
+				
+				release_message = trigger_release_event
+					(&release_remote_name,
+					 &release_button_name);
+				if(release_message)
+				{
+					input_message(release_message,
+						      release_remote_name,
+						      release_button_name,
+						      0,
+						      1);
+				}
+			}
 			if(free_remotes!=NULL)
 			{
 				free_old_remotes();
@@ -1739,6 +2084,14 @@ int waitfordata(long maxusec)
 		}
 		while(ret==-1 && errno==EINTR);
 		
+		if(hw.fd == -1 &&
+		   use_hw() &&
+		   hw.init_func)
+		{
+			log_enable(0);
+			hw.init_func();
+			log_enable(1);
+		}
 		for(i=0;i<clin;i++)
 		{
 			if(FD_ISSET(clis[i],&fds))
@@ -1778,8 +2131,10 @@ int waitfordata(long maxusec)
 			LOGPRINTF(1,"registering inet client");
 			add_client(sockinet);
 		}
-                if(clin>0 && hw.rec_mode!=0 && FD_ISSET(hw.fd,&fds))
+                if(use_hw() && hw.rec_mode!=0 && hw.fd!=-1 &&
+		   FD_ISSET(hw.fd,&fds))
                 {
+			register_input();
                         /* we will read later */
 			return(1);
                 }
@@ -1789,9 +2144,8 @@ int waitfordata(long maxusec)
 void loop()
 {
 	char *message;
-	int len,i;
 	
-	logprintf(LOG_NOTICE,"lircd(%s) ready",LIRC_DRIVER);
+	logprintf(LOG_NOTICE,"lircd(%s) ready", hw.name);
 	while(1)
 	{
 		(void) waitfordata(0);
@@ -1800,17 +2154,20 @@ void loop()
 		
 		if(message!=NULL)
 		{
-			len=strlen(message);
+			const char *remote_name;
+			const char *button_name;
+			int reps;
 			
-			for (i=0; i<clin; i++)
+			if(hw.ioctl_func &&
+			   (hw.features&LIRC_CAN_NOTIFY_DECODE))
 			{
-				LOGPRINTF(1,"writing to client %d",i);
-				if(write_socket(clis[i],message,len)<len)
-				{
-					remove_client(clis[i]);
-					i--;
-				}			
+				hw.ioctl_func(LIRC_NOTIFY_DECODE, NULL);
 			}
+			
+			get_release_data(&remote_name, &button_name, &reps);
+
+			input_message(message, remote_name, button_name,
+				      reps, 0);
 		}
 	}
 }
@@ -1822,6 +2179,7 @@ int main(int argc,char **argv)
 	mode_t permission=S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH;
 	char *device=NULL;
 
+	address.s_addr = htonl(INADDR_ANY);
 	hw_choose_driver(NULL);
 	while(1)
 	{
@@ -1843,10 +2201,18 @@ int main(int argc,char **argv)
 #                       endif
 #                       ifdef DEBUG
 			{"debug",optional_argument,NULL,'D'},
+#                       endif
+			{"release",optional_argument,NULL,'r'},
+			{"allow-simulate",no_argument,NULL,'a'},
+#                       if defined(__linux__)
+			{"uinput",no_argument,NULL,'u'},
 #                       endif
 			{0, 0, 0, 0}
 		};
-		c = getopt_long(argc,argv,"hvnp:H:d:o:P:l::c:"
+		c = getopt_long(argc,argv,"hvnp:H:d:o:P:l::c:r::a"
+#                               if defined(__linux__)
+				"u"
+#                               endif
 #                               ifndef USE_SYSLOG
 				"L:"
 #                               endif
@@ -1866,7 +2232,7 @@ int main(int argc,char **argv)
 			printf("\t -p --permission=mode\t\tfile permissions for " LIRCD "\n");
 			printf("\t -H --driver=driver\t\tuse given driver\n");
 			printf("\t -d --device=device\t\tread from given device\n");
-			printf("\t -l --listen[=port]\t\tlisten for network connections on port\n");
+			printf("\t -l --listen[=[address:]port]\tlisten for network connections\n");
 			printf("\t -c --connect=host[:port]\tconnect to remote lircd server\n");
                         printf("\t -o --output=socket\t\toutput socket filename\n");
                         printf("\t -P --pidfile=file\t\tdaemon pid file\n");
@@ -1875,10 +2241,15 @@ int main(int argc,char **argv)
 #                       endif
 #                       ifdef DEBUG
 			printf("\t -D[debug_level] --debug[=debug_level]\n");
+#                       endif
+			printf("\t -r --release[=suffix]\t\tauto-generate release events\n");
+			printf("\t -a --allow-simulate\t\taccept SIMULATE command\n");
+#                       if defined(__linux__)
+			printf("\t -u --uinput\t\tgenerate Linux input events\n");
 #                       endif
 			return(EXIT_SUCCESS);
 		case 'v':
-			printf("%s\n",progname);
+			printf("%s %s\n",progname,VERSION);
 			return(EXIT_SUCCESS);
 		case 'n':
 			nodaemon=1;
@@ -1919,16 +2290,28 @@ int main(int argc,char **argv)
 			{
 				long p;
 				char *endptr;
-				
-				p=strtol(optarg,&endptr,10);
+				char *sep = strchr(optarg, ':');
+				char *port_str = sep ? sep + 1 : optarg;
+				p=strtol(port_str,&endptr,10);
 				if(!*optarg || *endptr || p<1 || p>USHRT_MAX)
 				{
 					fprintf(stderr,
 						"%s: bad port number \"%s\"\n",
-						progname,optarg);
+						progname,port_str);
 					return(EXIT_FAILURE);
 				}
 				port=(unsigned short int) p;
+				if (sep)
+				{
+					*sep = 0;
+					if(!inet_aton(optarg, &address))
+					{
+						fprintf(stderr,
+							"%s: bad address \"%s\"\n",
+       							progname,optarg);
+						return(EXIT_FAILURE);
+					}
+				}
 			}
 			else
 			{
@@ -1948,6 +2331,25 @@ int main(int argc,char **argv)
 				debug=atoi(optarg);
 			}
 			break;
+#               endif
+		case 'r':
+			if(optarg)
+			{
+				set_release_suffix(optarg);
+			}
+			else
+			{
+				set_release_suffix(LIRC_RELEASE_SUFFIX);
+			}
+			userelease=1;
+			break;
+		case 'a':
+			allow_simulate=1;
+			break;
+#               if defined(__linux__)
+		case 'u':
+			useuinput=1;
+			break;
 #               endif
 		default:
 			printf("Usage: %s [options] [config-file]\n",progname);
@@ -1974,7 +2376,7 @@ int main(int argc,char **argv)
 			"no peers are specified\n",progname);
 		return(EXIT_FAILURE);
 	}
-	if(strcmp(hw.device, lircdfile)==0)
+	if(hw.device!=NULL && strcmp(hw.device, lircdfile)==0)
 	{
 		fprintf(stderr, "%s: refusing to connect to myself\n",
 			progname);
@@ -2030,13 +2432,14 @@ int main(int argc,char **argv)
 			{
 				repeat_remote=NULL;
 				repeat_code=NULL;
-				hw.send_func(r,c);
+				c->transmit_state = NULL;
+				send_ir_ncode(r,c);
 				repeat_remote=r;
 				repeat_code=c;
-				hw.send_func(r,c);
-				hw.send_func(r,c);
-				hw.send_func(r,c);
-				hw.send_func(r,c);
+				send_ir_ncode(r,c);
+				send_ir_ncode(r,c);
+				send_ir_ncode(r,c);
+				send_ir_ncode(r,c);
 				c++;
 			}
 			r=r->next;
