@@ -1,4 +1,4 @@
-/*      $Id: lircmd.c,v 5.17 2005/03/11 18:39:24 lirc Exp $      */
+/*      $Id: lircmd.c,v 5.19 2006/10/09 07:22:14 lirc Exp $      */
 
 /****************************************************************************
  ** lircmd.c ****************************************************************
@@ -135,7 +135,7 @@ struct state_mouse new_ms,ms=
 	{button_up,button_up,button_up}
 };
 
-char *progname="lircmd "VERSION;
+char *progname="lircmd";
 static const char *syslogident = "lircmd-" VERSION;
 char *configfile=LIRCMDCFGFILE;
 
@@ -460,7 +460,7 @@ void mouse_conv(int rep,char *button,char *remote)
 #endif
 							mouse_button(0,up,rep);
 						}
-						else if(down || up);
+						else if(down || up)
 						{
 							mouse_button(down,up,rep);
 						}
@@ -739,7 +739,7 @@ int main(int argc,char **argv)
 			printf("\t -n --nodaemon\t\tdon't fork to background\n");
 			return(EXIT_SUCCESS);
 		case 'v':
-			printf("%s\n",progname);
+			printf("%s %s\n",progname,VERSION);
 			return(EXIT_SUCCESS);
 		case 'n':
 			nodaemon=1;
