@@ -1,4 +1,4 @@
-/*      $Id: receive.c,v 5.28 2005/06/18 20:39:45 lirc Exp $      */
+/*      $Id: receive.c,v 5.36 2009/04/26 09:20:21 lirc Exp $      */
 
 /****************************************************************************
  ** receive.c ***************************************************************
@@ -32,11 +32,13 @@ inline lirc_t lirc_t_max(lirc_t a,lirc_t b)
 
 inline void set_pending_pulse(lirc_t deltap)
 {
+	LOGPRINTF(5, "pending pulse: %lu", deltap);
 	rec_buffer.pendingp=deltap;
 }
 
 inline void set_pending_space(lirc_t deltas)
 {
+	LOGPRINTF(5, "pending space: %lu", deltas);
 	rec_buffer.pendings=deltas;
 }
 
@@ -57,7 +59,7 @@ lirc_t get_next_rec_buffer(lirc_t maxusec)
 		{
 			lirc_t data;
 			
-			data=hw.readdata(2*maxusec<50000 ? 50000:2*maxusec);
+			data=hw.readdata(2*maxusec<100000 ? 100000:2*maxusec);
 			if(!data)
 			{
 				LOGPRINTF(3,"timeout: %u", maxusec);
@@ -168,6 +170,7 @@ int clear_rec_buffer(void)
 
 inline void unget_rec_buffer(int count)
 {
+	LOGPRINTF(5, "unget: %d", count);
 	if(count==1 || count==2)
 	{
 		rec_buffer.rptr-=count;
@@ -248,6 +251,7 @@ int expectpulse(struct ir_remote *remote,int exdelta)
 	lirc_t deltap;
 	int retval;
 	
+	LOGPRINTF(5, "expecting pulse: %lu", exdelta);
 	if(!sync_pending_space(remote)) return 0;
 	
 	deltap=get_next_pulse(rec_buffer.pendingp+exdelta);
@@ -271,6 +275,7 @@ int expectspace(struct ir_remote *remote,int exdelta)
 	lirc_t deltas;
 	int retval;
 
+	LOGPRINTF(5, "expecting space: %lu", exdelta);
 	if(!sync_pending_pulse(remote)) return 0;
 	
 	deltas=get_next_space(rec_buffer.pendings+exdelta);
@@ -293,9 +298,7 @@ inline int expectone(struct ir_remote *remote,int bit)
 {
 	if(is_biphase(remote))
 	{
-		int all_bits = remote->pre_data_bits+
-			remote->bits+
-			remote->post_data_bits;
+		int all_bits = bit_count(remote);
 		ir_code mask;
 		
 		mask=((ir_code) 1)<<(all_bits-1-bit);
@@ -360,9 +363,7 @@ inline int expectzero(struct ir_remote *remote,int bit)
 {
 	if(is_biphase(remote))
 	{
-		int all_bits = remote->pre_data_bits+
-			remote->bits+
-			remote->post_data_bits;
+		int all_bits = bit_count(remote);
 		ir_code mask;
 		
 		mask=((ir_code) 1)<<(all_bits-1-bit);
@@ -433,7 +434,7 @@ inline lirc_t sync_rec_buffer(struct ir_remote *remote)
 	if(last_remote!=NULL && !is_rcmm(remote))
 	{
 		while(!expect_at_least(last_remote, deltas,
-				       last_remote->remaining_gap))
+				       last_remote->min_remaining_gap))
 		{
 			deltap=get_next_pulse(1000000);
 			if(deltap==0) return(0);
@@ -449,7 +450,7 @@ inline lirc_t sync_rec_buffer(struct ir_remote *remote)
 		if(has_toggle_mask(remote))
 		{
 			if(!expect_at_most(last_remote, deltas,
-					   last_remote->remaining_gap))
+					   last_remote->max_remaining_gap))
 			{
 				remote->toggle_mask_state=0;
 				remote->toggle_code=NULL;
@@ -487,6 +488,19 @@ inline int get_header(struct ir_remote *remote)
 		unget_rec_buffer(2);
 		return(0);
 	}
+	else if(is_bo(remote))
+	{
+		if(expectpulse(remote, remote->pone) &&
+		   expectspace(remote, remote->sone) &&
+		   expectpulse(remote, remote->pone) &&
+		   expectspace(remote, remote->sone) &&
+		   expectpulse(remote, remote->phead) &&
+		   expectspace(remote, remote->shead))
+		{
+			return 1;
+		}
+		return 0;
+	}
 	if(remote->shead==0)
 	{
 		if(!sync_pending_space(remote)) return 0;
@@ -553,7 +567,7 @@ inline int get_gap(struct ir_remote *remote,lirc_t gap)
 	lirc_t data;
 	
 	LOGPRINTF(2,"sum: %d",rec_buffer.sum);
-	data=get_next_rec_buffer(gap*(100-remote->eps)/100);
+	data=get_next_rec_buffer(gap-gap*remote->eps/100);
 	if(data==0) return(1);
 	if(!is_space(data))
 	{
@@ -585,8 +599,8 @@ inline int get_repeat(struct ir_remote *remote)
 	if(!get_trail(remote)) return(0);
 	if(!get_gap(remote,
 		    is_const(remote) ? 
-		    (remote->gap>rec_buffer.sum ? remote->gap-rec_buffer.sum:0):
-		    (has_repeat_gap(remote) ? remote->repeat_gap:remote->gap)
+		    (min_gap(remote)>rec_buffer.sum ? min_gap(remote)-rec_buffer.sum:0):
+		    (has_repeat_gap(remote) ? remote->repeat_gap:min_gap(remote))
 		    )) return(0);
 	return(1);
 }
@@ -607,6 +621,7 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 			logprintf(LOG_ERR,"invalid bit number.");
 			return((ir_code) -1);
 		}
+		if(!sync_pending_space(remote)) return 0;
 		for(i=0;i<bits;i+=2)
 		{
 			code<<=2;
@@ -893,6 +908,68 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 		set_pending_space(0);
 		return code;
 	}
+	else if(is_bo(remote))
+	{
+		int lastbit = 1;
+		lirc_t deltap,deltas;
+		lirc_t pzero,szero;
+		lirc_t pone,sone;
+		
+		for(i=0; i<bits; i++)
+		{
+			code<<=1;
+			deltap=get_next_pulse(remote->pzero+remote->pone+
+					      remote->ptwo+remote->pthree);
+			deltas=get_next_space(remote->szero+remote->sone+
+					      remote->stwo+remote->sthree);
+			if(deltap==0 || deltas==0) 
+			{
+				logprintf(LOG_ERR,"failed on bit %d",
+					  done+i+1);
+				return((ir_code) -1);
+			}
+			if(lastbit == 1)
+			{
+				pzero = remote->pone;
+				szero = remote->sone;
+				pone = remote->ptwo;
+				sone = remote->stwo;
+			}
+			else
+			{
+				pzero = remote->ptwo;
+				szero = remote->stwo;
+				pone = remote->pthree;
+				sone = remote->sthree;
+			}
+			LOGPRINTF(5, "%lu %lu %lu %lu", pzero, szero, pone, sone);
+			if(expect(remote, deltap, pzero))
+			{
+				if(expect(remote, deltas, szero))
+				{
+					code|=0;
+					lastbit=0;
+					LOGPRINTF(2,"0");
+					continue;
+				}
+			}
+			
+			if(expect(remote, deltap, pone))
+			{
+				if(expect(remote, deltas, sone))
+				{
+					code|=1;
+					lastbit=1;
+					LOGPRINTF(2,"1");
+					continue;
+				}
+			}
+			logprintf(LOG_ERR,"failed on bit %d",
+				  done+i+1);
+			return((ir_code) -1);
+		}
+		return code;
+	}
 	
 	for(i=0;i<bits;i++)
 	{
@@ -976,7 +1053,8 @@ ir_code get_post(struct ir_remote *remote)
 
 int receive_decode(struct ir_remote *remote,
 		   ir_code *prep,ir_code *codep,ir_code *postp,
-		   int *repeat_flagp,lirc_t *remaining_gapp)
+		   int *repeat_flagp,
+		   lirc_t *min_remaining_gapp, lirc_t *max_remaining_gapp)
 {
 	ir_code pre,code,post,code_mask=0,post_mask=0;
 	lirc_t sync;
@@ -1030,12 +1108,18 @@ int receive_decode(struct ir_remote *remote,
 				*postp=remote->post_data;
 				*repeat_flagp=1;
 
-				*remaining_gapp=
-				is_const(remote) ? 
-				(remote->gap>rec_buffer.sum ?
-				 remote->gap-rec_buffer.sum:0):
-				(has_repeat_gap(remote) ?
-				 remote->repeat_gap:remote->gap);
+				*min_remaining_gapp=
+					is_const(remote) ? 
+					(min_gap(remote)>rec_buffer.sum ?
+					 min_gap(remote)-rec_buffer.sum:0):
+					(has_repeat_gap(remote) ?
+					 remote->repeat_gap:min_gap(remote));
+				*max_remaining_gapp=
+					is_const(remote) ? 
+					(max_gap(remote)>rec_buffer.sum ?
+					 max_gap(remote)-rec_buffer.sum:0):
+					(has_repeat_gap(remote) ?
+					 remote->repeat_gap:max_gap(remote));
 				return(1);
 			}
 			else
@@ -1054,7 +1138,7 @@ int receive_decode(struct ir_remote *remote,
 			{
 				header=0;
 				if(!(remote->flags&NO_HEAD_REP && 
-				     expect_at_most(remote,sync,remote->gap)))
+				     expect_at_most(remote,sync,max_gap(remote))))
 				{
 					LOGPRINTF(1,"failed on header");
 					return(0);
@@ -1102,8 +1186,8 @@ int receive_decode(struct ir_remote *remote,
 		{
 			if(!get_gap(remote,
 				    is_const(remote) ? 
-				    remote->gap-rec_buffer.sum:
-				    remote->gap)) 
+				    min_gap(remote)-rec_buffer.sum:
+				    min_gap(remote))) 
 				found=NULL;
 		}
 		if(found==NULL) return(0);
@@ -1123,12 +1207,10 @@ int receive_decode(struct ir_remote *remote,
 			LOGPRINTF(1,"decoded: %lx",rec_buffer.decoded);
 #                       endif
 			if((hw.rec_mode==LIRC_MODE_CODE &&
-			    hw.code_length<remote->pre_data_bits
-			    +remote->bits+remote->post_data_bits)
+			    hw.code_length<bit_count(remote))
 			   ||
 			   (hw.rec_mode==LIRC_MODE_LIRCCODE && 
-			    hw.code_length!=remote->pre_data_bits
-			    +remote->bits+remote->post_data_bits))
+			    hw.code_length!=bit_count(remote)))
 			{
 				return(0);
 			}
@@ -1152,9 +1234,7 @@ int receive_decode(struct ir_remote *remote,
 			sum=remote->phead+remote->shead+
 				lirc_t_max(remote->pone+remote->sone,
 					   remote->pzero+remote->szero)*
-				(remote->bits+
-				 remote->pre_data_bits+
-				 remote->post_data_bits)+
+				bit_count(remote)+
 				remote->plead+
 				remote->ptrail+
 				remote->pfoot+remote->sfoot+
@@ -1241,20 +1321,20 @@ int receive_decode(struct ir_remote *remote,
 			else if(is_const(remote))
 			{
 				if(!get_gap(remote,
-					    remote->gap>rec_buffer.sum ?
-					    remote->gap-rec_buffer.sum:0))
+					    min_gap(remote)>rec_buffer.sum ?
+					    min_gap(remote)-rec_buffer.sum:0))
 					return(0);
 			}
 			else
 			{
-				if(!get_gap(remote,remote->gap))
+				if(!get_gap(remote, min_gap(remote)))
 					return(0);
 			}
 		} /* end of mode specific code */
 	}
 	*prep=pre;*codep=code;*postp=post;
-	if(!has_repeat(remote) &&
-	   expect_at_most(remote, sync, remote->remaining_gap))
+	if((!has_repeat(remote) || remote->reps<remote->min_code_repeat) &&
+	   expect_at_most(remote, sync, remote->max_remaining_gap))
 		*repeat_flagp=1;
 	else
 		*repeat_flagp=0;
@@ -1271,12 +1351,15 @@ int receive_decode(struct ir_remote *remote,
 	}
 	if(is_const(remote))
 	{
-		*remaining_gapp=remote->gap>rec_buffer.sum ?
-			remote->gap-rec_buffer.sum:0;
+		*min_remaining_gapp=min_gap(remote)>rec_buffer.sum ?
+			min_gap(remote)-rec_buffer.sum:0;
+		*max_remaining_gapp=max_gap(remote)>rec_buffer.sum ?
+			max_gap(remote)-rec_buffer.sum:0;
 	}
 	else
 	{
-		*remaining_gapp=remote->gap;
+		*min_remaining_gapp=min_gap(remote);
+		*max_remaining_gapp=max_gap(remote);
 	}
 	return(1);
 }
