@@ -1,7 +1,15 @@
-/*      $Id: lirc_i2c.c,v 1.47 2008/08/12 20:50:39 lirc Exp $      */
+/*      $Id: lirc_i2c.c,v 1.64 2009/02/14 19:35:52 lirc Exp $      */
 
 /*
- * i2c IR lirc plugin for Hauppauge and Pixelview cards - new 2.3.x i2c stack
+ * lirc_i2c.c
+ *
+ * i2c IR driver for the onboard IR port on many TV tuner cards, including:
+ *  -Flavors of the Hauppauge PVR-150/250/350
+ *  -Hauppauge HVR-1300
+ *  -PixelView (BT878P+W/FM)
+ *  -KNC ONE TV Station/Anubis Typhoon TView Tuner
+ *  -Asus TV-Box and Creative/VisionTek BreakOut-Box
+ *  -Leadtek Winfast PVR2000
  *
  * Copyright (c) 2000 Gerd Knorr <kraxel@goldbach.in-berlin.de>
  * modified for PixelView (BT878P+W/FM) by
@@ -33,7 +41,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
  */
 
 
@@ -54,17 +61,11 @@
 #include <linux/i2c.h>
 #include <linux/i2c-algo-bit.h>
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 27)
-#include <asm/semaphore.h>
-#else
-#include <linux/semaphore.h>
-#endif
-
 #include "drivers/kcompat.h"
 #include "drivers/lirc_dev/lirc_dev.h"
 
 struct IR {
-	struct lirc_plugin l;
+	struct lirc_driver l;
 	struct i2c_client  c;
 	int nextkey;
 	unsigned char b[3];
@@ -72,13 +73,9 @@ struct IR {
 	unsigned char flag;
 };
 
-/* ----------------------------------------------------------------------- */
-
 #define DEVICE_NAME "lirc_i2c"
 
-/* ----------------------------------------------------------------------- */
-/* insmod parameters						       */
-
+/* module parameters */
 static int debug;	/* debug output */
 static int minor = -1;	/* minor number */
 
@@ -89,15 +86,13 @@ static int minor = -1;	/* minor number */
 			       ## args);				\
 	} while (0)
 
-/* ----------------------------------------------------------------------- */
-
-static inline int reverse(int data, int bits)
+static int reverse(int data, int bits)
 {
 	int i;
 	int c;
 
 	for (c = 0, i = 0; i < bits; i++)
-		c |= (((data & (1<<i)) ? 1:0)) << (bits-1-i);
+		c |= ((data & (1<<i)) ? 1 : 0) << (bits-1-i);
 
 	return c;
 }
@@ -126,7 +121,7 @@ static int add_to_buf_adap(void *data, struct lirc_buffer *buf)
 	keybuf[2] &= 0x7f;
 	keybuf[3] |= 0x80;
 
-	lirc_buffer_write_1(buf, keybuf);
+	lirc_buffer_write(buf, keybuf);
 	return 0;
 }
 
@@ -163,14 +158,13 @@ static int add_to_buf_pcf8574(void *data, struct lirc_buffer *buf)
 	dprintk("%s key 0x%02X %s\n", ir->c.name, rc & ir->bits,
 		(rc & ir->flag) ? "released" : "pressed");
 
-	if (rc & ir->flag) {
-		/* ignore released buttons */
+	/* ignore released buttons */
+	if (rc & ir->flag)
 		return -ENODATA;
-	}
 
 	/* set valid key code */
 	key  = rc & ir->bits;
-	lirc_buffer_write_1(buf, &key);
+	lirc_buffer_write(buf, &key);
 	return 0;
 }
 
@@ -190,7 +184,7 @@ static int add_to_buf_haup_common(void *data, struct lirc_buffer *buf,
 		dprintk("key (0x%02x/0x%02x)\n", ir->b[0], ir->b[1]);
 	} else {
 		dprintk("read error\n");
-		/* keep last successfull read buffer */
+		/* keep last successful read buffer */
 	}
 
 	/* key pressed ? */
@@ -204,7 +198,7 @@ static int add_to_buf_haup_common(void *data, struct lirc_buffer *buf,
 	codes[1] = code & 0xff;
 
 	/* return it */
-	lirc_buffer_write_1(buf, codes);
+	lirc_buffer_write(buf, codes);
 	return 0;
 }
 
@@ -254,7 +248,7 @@ static int add_to_buf_pvr2000(void *data, struct lirc_buffer *buf)
 	dprintk("IR Key/Flags: (0x%02x/0x%02x)\n", key, flags & 0xFF);
 
 	/* return it */
-	lirc_buffer_write_1(buf, &key);
+	lirc_buffer_write(buf, &key);
 	return 0;
 }
 
@@ -271,7 +265,7 @@ static int add_to_buf_pixelview(void *data, struct lirc_buffer *buf)
 	dprintk("key %02x\n", key);
 
 	/* return it */
-	lirc_buffer_write_1(buf, &key);
+	lirc_buffer_write(buf, &key);
 	return 0;
 }
 
@@ -296,7 +290,7 @@ static int add_to_buf_pv951(void *data, struct lirc_buffer *buf)
 	codes[2] = reverse(key, 8);
 	codes[3] = (~codes[2])&0xff;
 
-	lirc_buffer_write_1(buf, codes);
+	lirc_buffer_write(buf, codes);
 	return 0;
 }
 
@@ -312,9 +306,11 @@ static int add_to_buf_knc1(void *data, struct lirc_buffer *buf)
 		return -ENODATA;
 	}
 
-	/* it seems that 0xFE indicates that a button is still hold
-	   down, while 0xFF indicates that no button is hold
-	   down. 0xFE sequences are sometimes interrupted by 0xFF */
+	/*
+	 * it seems that 0xFE indicates that a button is still held
+	 * down, while 0xFF indicates that no button is held
+	 * down. 0xFE sequences are sometimes interrupted by 0xFF
+	 */
 
 	dprintk("key %02x\n", key);
 
@@ -325,7 +321,7 @@ static int add_to_buf_knc1(void *data, struct lirc_buffer *buf)
 		key = last_key;
 
 	last_key = key;
-	lirc_buffer_write_1(buf, &key);
+	lirc_buffer_write(buf, &key);
 
 	return 0;
 }
@@ -357,7 +353,7 @@ static void set_use_dec(void *data)
 	MOD_DEC_USE_COUNT;
 }
 
-static struct lirc_plugin lirc_template = {
+static struct lirc_driver lirc_template = {
 	.name		= "lirc_i2c",
 	.set_use_inc	= set_use_inc,
 	.set_use_dec	= set_use_dec,
@@ -365,8 +361,6 @@ static struct lirc_plugin lirc_template = {
 	.owner		= THIS_MODULE,
 };
 
-/* ----------------------------------------------------------------------- */
-
 static int ir_attach(struct i2c_adapter *adap, int addr,
 		      unsigned short flags, int kind);
 static int ir_detach(struct i2c_client *client);
@@ -398,7 +392,7 @@ static int ir_attach(struct i2c_adapter *adap, int addr,
 		     unsigned short flags, int kind)
 {
 	struct IR *ir;
-	int err;
+	int err, retval;
 
 	client_template.adapter = adap;
 	client_template.addr = addr;
@@ -406,7 +400,7 @@ static int ir_attach(struct i2c_adapter *adap, int addr,
 	ir = kmalloc(sizeof(struct IR), GFP_KERNEL);
 	if (!ir)
 		return -ENOMEM;
-	memcpy(&ir->l, &lirc_template, sizeof(struct lirc_plugin));
+	memcpy(&ir->l, &lirc_template, sizeof(struct lirc_driver));
 	memcpy(&ir->c, &client_template, sizeof(struct i2c_client));
 
 	ir->c.adapter = adap;
@@ -415,6 +409,7 @@ static int ir_attach(struct i2c_adapter *adap, int addr,
 	ir->l.data    = ir;
 	ir->l.minor   = minor;
 	ir->l.sample_rate = 10;
+	ir->l.dev     = &ir->c.dev;
 	ir->nextkey   = -1;
 
 	switch (addr) {
@@ -435,9 +430,11 @@ static int ir_attach(struct i2c_adapter *adap, int addr,
 #else
 		if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_BT848)) {
 #endif
-			/* The PVR150 IR receiver uses the same protocol as
+			/*
+			 * The PVR150 IR receiver uses the same protocol as
 			 * other Hauppauge cards, but the data flow is
-			 * different, so we need to deal with it by its own. */
+			 * different, so we need to deal with it by its own.
+			 */
 			strlcpy(ir->c.name, "Hauppauge PVR150", I2C_NAME_SIZE);
 		} else /* I2C_HW_B_CX2388x */
 			strlcpy(ir->c.name, "Hauppauge HVR1300", I2C_NAME_SIZE);
@@ -483,7 +480,7 @@ static int ir_attach(struct i2c_adapter *adap, int addr,
 		/* shouldn't happen */
 		printk("lirc_i2c: Huh? unknown i2c address (0x%02x)?\n", addr);
 		kfree(ir);
-		return -1;
+		return -EINVAL;
 	}
 	printk(KERN_INFO "lirc_i2c: chip 0x%x found @ 0x%02x (%s)\n",
 	       adap->id, addr, ir->c.name);
@@ -494,7 +491,17 @@ static int ir_attach(struct i2c_adapter *adap, int addr,
 		kfree(ir);
 		return err;
 	}
-	ir->l.minor = lirc_register_plugin(&ir->l);
+
+	retval = lirc_register_driver(&ir->l);
+
+	if (retval < 0) {
+		printk(KERN_ERR "lirc_i2c: failed to register driver!\n");
+		i2c_detach_client(&ir->c);
+		kfree(ir);
+		return retval;
+	}
+
+	ir->l.minor = retval;
 
 	return 0;
 }
@@ -504,7 +511,7 @@ static int ir_detach(struct i2c_client *client)
 	struct IR *ir = i2c_get_clientdata(client);
 
 	/* unregister device */
-	lirc_unregister_plugin(ir->l.minor);
+	lirc_unregister_driver(ir->l.minor);
 	i2c_detach_client(&ir->c);
 
 	/* free memory */
@@ -514,7 +521,8 @@ static int ir_detach(struct i2c_client *client)
 
 static int ir_probe(struct i2c_adapter *adap)
 {
-	/* The external IR receiver is at i2c address 0x34 (0x35 for
+	/*
+	 * The external IR receiver is at i2c address 0x34 (0x35 for
 	 * reads).  Future Hauppauge cards will have an internal
 	 * receiver at 0x30 (0x31 for reads).  In theory, both can be
 	 * fitted, and Hauppauge suggest an external overrides an
@@ -523,7 +531,8 @@ static int ir_probe(struct i2c_adapter *adap)
 	 * That's why we probe 0x1a (~0x34) first. CB
 	 *
 	 * The i2c address for the Hauppauge PVR-150 card is 0xe2,
-	 * so we need to probe 0x71 as well. */
+	 * so we need to probe 0x71 as well.
+	 */
 
 	static const int probe[] = {
 		0x1a, /* Hauppauge IR external */
@@ -546,14 +555,13 @@ static int ir_probe(struct i2c_adapter *adap)
 	char buf;
 	int i, rc;
 
+	memset(&c, 0, sizeof(c));
 #ifdef I2C_HW_B_CX2341X
 	if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_BT848) ||
-	    adap->id == (I2C_ALGO_BIT | I2C_HW_B_CX2341X))
+	    adap->id == (I2C_ALGO_BIT | I2C_HW_B_CX2341X)) {
 #else
-	if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_BT848))
+	if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_BT848)) {
 #endif
-	{
-		memset(&c, 0, sizeof(c));
 		c.adapter = adap;
 		for (i = 0; -1 != probe[i]; i++) {
 			c.addr = probe[i];
@@ -561,15 +569,17 @@ static int ir_probe(struct i2c_adapter *adap)
 			dprintk("probe 0x%02x @ %s: %s\n",
 				probe[i], adap->name,
 				(1 == rc) ? "yes" : "no");
-			if (1 == rc)
-				ir_attach(adap, probe[i], 0, 0);
+			if (1 == rc) {
+				rc = ir_attach(adap, probe[i], 0, 0);
+				if (rc < 0)
+					goto attach_fail;
+			}
 		}
 	}
 
 #ifdef I2C_HW_B_CX2388x
 	/* Leadtek Winfast PVR2000 or Hauppauge HVR-1300 */
 	else if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_CX2388x)) {
-		memset(&c, 0, sizeof(c));
 		c.adapter = adap;
 		for (i = 0; -1 != probe_cx88[i]; i++) {
 			c.addr = probe_cx88[i];
@@ -577,27 +587,30 @@ static int ir_probe(struct i2c_adapter *adap)
 			dprintk("probe 0x%02x @ %s: %s\n",
 				c.addr, adap->name,
 				(1 == rc) ? "yes" : "no");
-			if (1 == rc)
-				ir_attach(adap, c.addr, 0, 0);
+			if (1 == rc) {
+				rc = ir_attach(adap, c.addr, 0, 0);
+				if (rc < 0)
+					goto attach_fail;
+			}
 		}
 	}
 #endif
 
 	/* Asus TV-Box and Creative/VisionTek BreakOut-Box (PCF8574) */
 	else if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_RIVA)) {
-		/* addresses to probe;
-		   leave 0x24 and 0x25 because SAA7113H possibly uses it
-		   0x21 and 0x22 possibly used by SAA7108E
-		   Asus:      0x21 is a correct address (channel 1 of PCF8574)
-		   Creative:  0x23 is a correct address (channel 3 of PCF8574)
-		   VisionTek: 0x23 is a correct address (channel 3 of PCF8574)
-		*/
+		/*
+		 * addresses to probe;
+		 * leave 0x24 and 0x25 because SAA7113H possibly uses it
+		 * 0x21 and 0x22 possibly used by SAA7108E
+		 * Asus:      0x21 is a correct address (channel 1 of PCF8574)
+		 * Creative:  0x23 is a correct address (channel 3 of PCF8574)
+		 * VisionTek: 0x23 is a correct address (channel 3 of PCF8574)
+		 */
 		static const int pcf_probe[] = { 0x20, 0x21, 0x22, 0x23,
 						 0x24, 0x25, 0x26, 0x27, -1 };
 		int ret1, ret2, ret3, ret4;
 		unsigned char bits = 0, flag = 0;
 
-		memset(&c, 0, sizeof(c));
 		c.adapter = adap;
 		for (i = 0; -1 != pcf_probe[i]; i++) {
 			c.addr = pcf_probe[i];
@@ -627,13 +640,21 @@ static int ir_probe(struct i2c_adapter *adap)
 			}
 			dprintk("probe 0x%02x @ %s: %s\n",
 				c.addr, adap->name, rc ? "yes" : "no");
-			if (rc)
-				ir_attach(adap, pcf_probe[i],
-					  bits|(flag<<8), 0);
+			if (rc) {
+				rc = ir_attach(adap, pcf_probe[i],
+					       bits | (flag << 8), 0);
+				if (rc < 0)
+					goto attach_fail;
+			}
 		}
 	}
 
 	return 0;
+
+attach_fail:
+	printk(KERN_ERR "lirc_i2c: %s: ir_attach failed!\n", __func__);
+	return rc;
+
 }
 
 static int ir_command(struct i2c_client *client, unsigned int cmd, void *arg)
@@ -642,10 +663,9 @@ static int ir_command(struct i2c_client *client, unsigned int cmd, void *arg)
 	return 0;
 }
 
-/* ----------------------------------------------------------------------- */
 #ifdef MODULE
 
-int init_module(void)
+static int __init lirc_i2c_init(void)
 {
 	request_module("bttv");
 	request_module("rivatv");
@@ -655,7 +675,7 @@ int init_module(void)
 	return 0;
 }
 
-void cleanup_module(void)
+static void __exit lirc_i2c_exit(void)
 {
 	i2c_del_driver(&driver);
 }
@@ -666,20 +686,14 @@ MODULE_AUTHOR("Gerd Knorr, Michal Kochanowicz, Christoph Bartelmus, "
 	      "Ulrich Mueller, Stefan Jahn, Jerome Brock");
 MODULE_LICENSE("GPL");
 
-module_param(minor, int, 0444);
+module_param(minor, int, S_IRUGO);
 MODULE_PARM_DESC(minor, "Preferred minor device number");
 
-module_param(debug, bool, 0644);
+module_param(debug, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Enable debugging messages");
 
+module_init(lirc_i2c_init);
+module_exit(lirc_i2c_exit);
 EXPORT_NO_SYMBOLS;
 
 #endif /* MODULE */
-
-/*
- * Overrides for Emacs so that we follow Linus's tabbing style.
- * ---------------------------------------------------------------------------
- * Local variables:
- * c-basic-offset: 8
- * End:
- */
