@@ -22,18 +22,17 @@
  * 2000/09/16 Frank Przybylski <mail@frankprzybylski.de> :
  *  added timeout and relaxed pulse detection, removed gap bug
  *
- * 2000/12/15 Christoph Bartelmus <lirc@bartelmus.de> : 
+ * 2000/12/15 Christoph Bartelmus <lirc@bartelmus.de> :
  *   added support for Tekram Irmate 210 (sending does not work yet,
  *   kind of disappointing that nobody was able to implement that
  *   before),
  *   major clean-up
  *
- * 2001/02/27 Christoph Bartelmus <lirc@bartelmus.de> : 
+ * 2001/02/27 Christoph Bartelmus <lirc@bartelmus.de> :
  *   added support for StrongARM SA1100 embedded microprocessor
  *   parts cut'n'pasted from sa1100_ir.c (C) 2000 Russell King
  */
 
-
 #include <linux/version.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 18)
 #error "**********************************************************"
@@ -45,10 +44,10 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
- 
-#include <linux/config.h>
 
-#if !defined(LIRC_ON_SA1100) && !defined(CONFIG_SERIAL_MODULE)
+#include <linux/autoconf.h>
+#if !defined(CONFIG_SERIAL_MODULE)
+#if !defined(LIRC_ON_SA1100)
 #warning "******************************************"
 #warning " Your serial port driver is compiled into "
 #warning " the kernel. You will have to release the "
@@ -56,8 +55,8 @@
 #warning "    setserial /dev/ttySx uart none        "
 #warning "******************************************"
 #endif
+#endif
 
-#include <linux/module.h>
 #include <linux/sched.h>
 #include <linux/errno.h>
 #include <linux/signal.h>
@@ -65,7 +64,6 @@
 #include <linux/interrupt.h>
 #include <linux/ioport.h>
 #include <linux/kernel.h>
-#include <linux/major.h>
 #include <linux/serial_reg.h>
 #include <linux/time.h>
 #include <linux/string.h>
@@ -75,10 +73,16 @@
 #include <linux/delay.h>
 #include <linux/poll.h>
 #include <asm/system.h>
-#include <asm/segment.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
+#include <asm/uaccess.h>
+#endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
 #include <asm/io.h>
+#else
+#include <linux/io.h>
+#endif
 #include <asm/irq.h>
-#include <asm/fcntl.h>
+#include <linux/fcntl.h>
 #ifdef LIRC_ON_SA1100
 #include <asm/hardware.h>
 #ifdef CONFIG_SA1100_COLLIE
@@ -95,7 +99,7 @@
 
 /* SECTION: Definitions */
 
-/**************************** Tekram dongle ***************************/
+/*** Tekram dongle ***/
 #ifdef LIRC_SIR_TEKRAM
 /* stolen from kernel source */
 /* definitions for Tekram dongle */
@@ -108,19 +112,20 @@
 
 #define TEKRAM_PW 0x10 /* Pulse select bit */
 
-/* 10bit * 1s/115200bit in milli seconds = 87ms*/
+/* 10bit * 1s/115200bit in milliseconds = 87ms*/
 #define TIME_CONST (10000000ul/115200ul)
 
 #endif
 
 #ifdef LIRC_SIR_ACTISYS_ACT200L
 static void init_act200(void);
+#elif defined(LIRC_SIR_ACTISYS_ACT220L)
+static void init_act220(void);
 #endif
 
-/******************************* SA1100 ********************************/
+/*** SA1100 ***/
 #ifdef LIRC_ON_SA1100
-struct sa1100_ser2_registers
-{
+struct sa1100_ser2_registers {
 	/* HSSP control register */
 	unsigned char hscr0;
 	/* UART registers */
@@ -134,7 +139,7 @@ struct sa1100_ser2_registers
 	unsigned char utsr1;
 } sr;
 
-static int irq=IRQ_Ser2ICP;
+static int irq = IRQ_Ser2ICP;
 
 #define LIRC_ON_SA1100_TRANSMITTER_LATENCY 0
 
@@ -152,16 +157,15 @@ static unsigned int duty_cycle = 50;   /* duty cycle of 50% */
 
 #define LIRC_DRIVER_NAME "lirc_sir"
 
-#ifndef LIRC_SIR_TEKRAM
 #define PULSE '['
 
+#ifndef LIRC_SIR_TEKRAM
 /* 9bit * 1s/115200bit in milli seconds = 78.125ms*/
 #define TIME_CONST (9000000ul/115200ul)
 #endif
 
 
-/* timeout for sequences in jiffies (=5/100s) */
-/* must be longer than TIME_CONST */
+/* timeout for sequences in jiffies (=5/100s), must be longer than TIME_CONST */
 #define SIR_TIMEOUT	(HZ*5/100)
 
 #ifndef LIRC_ON_SA1100
@@ -177,79 +181,80 @@ static int irq = LIRC_IRQ;
 static int threshold = 3;
 #endif
 
-static spinlock_t timer_lock = SPIN_LOCK_UNLOCKED;
+static DEFINE_SPINLOCK(timer_lock);
 static struct timer_list timerlist;
 /* time of last signal change detected */
 static struct timeval last_tv = {0, 0};
 /* time of last UART data ready interrupt */
 static struct timeval last_intr_tv = {0, 0};
-static int last_value = 0;
+static int last_value;
 
 static DECLARE_WAIT_QUEUE_HEAD(lirc_read_queue);
 
-static spinlock_t hardware_lock = SPIN_LOCK_UNLOCKED;
-static spinlock_t dev_lock = SPIN_LOCK_UNLOCKED;
+static DEFINE_SPINLOCK(hardware_lock);
+static DEFINE_SPINLOCK(dev_lock);
 
-static lirc_t rx_buf[RBUF_LEN]; unsigned int rx_tail = 0, rx_head = 0;
-#ifndef LIRC_SIR_TEKRAM
+static lirc_t rx_buf[RBUF_LEN];
+static unsigned int rx_tail, rx_head;
 static lirc_t tx_buf[WBUF_LEN];
-#endif
 
-static int debug = 0;
-#define dprintk(fmt, args...)                                     \
-	do{                                                       \
-		if(debug) printk(KERN_DEBUG LIRC_DRIVER_NAME ": " \
-                                 fmt, ## args);                   \
-	}while(0)
+static int debug;
+#define dprintk(fmt, args...)						\
+	do {								\
+		if (debug)						\
+			printk(KERN_DEBUG LIRC_DRIVER_NAME ": "		\
+				fmt, ## args);				\
+	} while (0)
 
 /* SECTION: Prototypes */
 
 /* Communication with user-space */
-static int lirc_open(struct inode * inode, struct file * file);
-static int lirc_close(struct inode * inode, struct file *file);
-static unsigned int lirc_poll(struct file * file, poll_table * wait);
-static ssize_t lirc_read(struct file * file, char * buf, size_t count,
-		loff_t * ppos);
-static ssize_t lirc_write(struct file * file, const char * buf, size_t n, loff_t * pos);
-static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
+static int lirc_open(struct inode *inode, struct file *file);
+static int lirc_close(struct inode *inode, struct file *file);
+static unsigned int lirc_poll(struct file *file, poll_table *wait);
+static ssize_t lirc_read(struct file *file, char *buf, size_t count,
+		loff_t *ppos);
+static ssize_t lirc_write(struct file *file, const char *buf, size_t n,
+		loff_t *pos);
+static int lirc_ioctl(struct inode *node, struct file *filep, unsigned int cmd,
 		unsigned long arg);
 static void add_read_queue(int flag, unsigned long val);
 #ifdef MODULE
 static int init_chrdev(void);
 static void drop_chrdev(void);
 #endif
-	/* Hardware */
-static irqreturn_t sir_interrupt(int irq, void * dev_id,
-				 struct pt_regs * regs);
-#ifndef LIRC_SIR_TEKRAM
+/* Hardware */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
+static irqreturn_t sir_interrupt(int irq, void *dev_id,
+				 struct pt_regs *regs);
+#else
+static irqreturn_t sir_interrupt(int irq, void *dev_id);
+#endif
 static void send_space(unsigned long len);
 static void send_pulse(unsigned long len);
-#endif
 static int init_hardware(void);
 static void drop_hardware(void);
-	/* Initialisation */
+/* Initialisation */
 static int init_port(void);
 static void drop_port(void);
-int init_module(void);
-void cleanup_module(void);
 
 #ifdef LIRC_ON_SA1100
-static void inline on(void)
+static void on(void)
 {
-	PPSR|=PPC_TXD2;
+	PPSR |= PPC_TXD2;
 }
-  
-static void inline off(void)
+
+static void off(void)
 {
-	PPSR&=~PPC_TXD2;
+	PPSR &= ~PPC_TXD2;
 }
 #else
-static inline unsigned int sinp(int offset)
+static unsigned int sinp(int offset)
 {
 	return inb(io + offset);
 }
 
-static inline void soutp(int offset, int value)
+static void soutp(int offset, int value)
 {
 	outb(value, io + offset);
 }
@@ -261,19 +266,18 @@ static inline void soutp(int offset, int value)
 #define MAX_UDELAY_US (MAX_UDELAY_MS*1000)
 #endif
 
-static inline void safe_udelay(unsigned long usecs)
+static void safe_udelay(unsigned long usecs)
 {
-	while(usecs>MAX_UDELAY_US)
-	{
+	while (usecs > MAX_UDELAY_US) {
 		udelay(MAX_UDELAY_US);
-		usecs-=MAX_UDELAY_US;
+		usecs -= MAX_UDELAY_US;
 	}
 	udelay(usecs);
 }
 
 /* SECTION: Communication with user-space */
 
-static int lirc_open(struct inode * inode, struct file * file)
+static int lirc_open(struct inode *inode, struct file *file)
 {
 	spin_lock(&dev_lock);
 	if (MOD_IN_USE) {
@@ -285,13 +289,13 @@ static int lirc_open(struct inode * inode, struct file * file)
 	return 0;
 }
 
-static int lirc_close(struct inode * inode, struct file *file)
+static int lirc_close(struct inode *inode, struct file *file)
 {
 	MOD_DEC_USE_COUNT;
 	return 0;
 }
 
-static unsigned int lirc_poll(struct file * file, poll_table * wait)
+static unsigned int lirc_poll(struct file *file, poll_table *wait)
 {
 	poll_wait(file, &lirc_read_queue, wait);
 	if (rx_head != rx_tail)
@@ -299,72 +303,60 @@ static unsigned int lirc_poll(struct file * file, poll_table * wait)
 	return 0;
 }
 
-static ssize_t lirc_read(struct file * file, char * buf, size_t count,
-		loff_t * ppos)
+static ssize_t lirc_read(struct file *file, char *buf, size_t count,
+		loff_t *ppos)
 {
-	int n=0;
+	int n = 0;
 	int retval = 0;
-	DECLARE_WAITQUEUE(wait,current);
-	
-	if(n%sizeof(lirc_t)) return(-EINVAL);
-	
-	add_wait_queue(&lirc_read_queue,&wait);
+	DECLARE_WAITQUEUE(wait, current);
+
+	if (count % sizeof(lirc_t))
+		return -EINVAL;
+
+	add_wait_queue(&lirc_read_queue, &wait);
 	set_current_state(TASK_INTERRUPTIBLE);
-	while(n<count)
-	{
-		if(rx_head!=rx_tail)
-		{
-			retval=verify_area(VERIFY_WRITE,
-					   (void *) buf+n,sizeof(lirc_t));
-			if (retval)
-			{
-				return retval;
-			}
-			copy_to_user((void *) buf+n,(void *) (rx_buf+rx_head),
-				     sizeof(lirc_t));
-			rx_head=(rx_head+1)&(RBUF_LEN-1);
-			n+=sizeof(lirc_t);
-		}
-		else
-		{
-			if(file->f_flags & O_NONBLOCK)
-			{
-				retval=-EAGAIN;
+	while (n < count) {
+		if (rx_head != rx_tail) {
+			if (copy_to_user((void *) buf + n,
+					(void *) (rx_buf + rx_head),
+					sizeof(lirc_t))) {
+				retval = -EFAULT;
 				break;
 			}
-			if(signal_pending(current))
-			{
-				retval=-ERESTARTSYS;
+			rx_head = (rx_head + 1) & (RBUF_LEN - 1);
+			n += sizeof(lirc_t);
+		} else {
+			if (file->f_flags & O_NONBLOCK) {
+				retval = -EAGAIN;
+				break;
+			}
+			if (signal_pending(current)) {
+				retval = -ERESTARTSYS;
 				break;
 			}
 			schedule();
 			set_current_state(TASK_INTERRUPTIBLE);
 		}
 	}
-	remove_wait_queue(&lirc_read_queue,&wait);
+	remove_wait_queue(&lirc_read_queue, &wait);
 	set_current_state(TASK_RUNNING);
-	return (n ? n : retval);
+	return n ? n : retval;
 }
-static ssize_t lirc_write(struct file * file, const char * buf, size_t n, loff_t * pos)
+static ssize_t lirc_write(struct file *file, const char *buf, size_t n,
+				loff_t *pos)
 {
 	unsigned long flags;
-#ifdef LIRC_SIR_TEKRAM
-	return(-EBADF);
-#else
 	int i;
-	int retval;
 
-        if(n%sizeof(lirc_t) || (n/sizeof(lirc_t)) > WBUF_LEN)
-		return(-EINVAL);
-	retval = verify_area(VERIFY_READ, buf, n);
-	if (retval)
-		return retval;
-	copy_from_user(tx_buf, buf, n);
+	if (n % sizeof(lirc_t) || (n / sizeof(lirc_t)) > WBUF_LEN)
+		return -EINVAL;
+	if (copy_from_user(tx_buf, buf, n))
+		return -EFAULT;
 	i = 0;
-	n/=sizeof(lirc_t);
+	n /= sizeof(lirc_t);
 #ifdef LIRC_ON_SA1100
 	/* disable receiver */
-	Ser2UTCR3=0;
+	Ser2UTCR3 = 0;
 #endif
 	local_irq_save(flags);
 	while (1) {
@@ -383,33 +375,23 @@ static ssize_t lirc_write(struct file * file, const char * buf, size_t n, loff_t
 #ifdef LIRC_ON_SA1100
 	off();
 	udelay(1000); /* wait 1ms for IR diode to recover */
-	Ser2UTCR3=0;
+	Ser2UTCR3 = 0;
 	/* clear status register to prevent unwanted interrupts */
 	Ser2UTSR0 &= (UTSR0_RID | UTSR0_RBB | UTSR0_REB);
 	/* enable receiver */
-	Ser2UTCR3=UTCR3_RXE|UTCR3_RIE;
+	Ser2UTCR3 = UTCR3_RXE|UTCR3_RIE;
 #endif
 	return n;
-#endif
 }
 
-static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
+static int lirc_ioctl(struct inode *node, struct file *filep, unsigned int cmd,
 		unsigned long arg)
 {
 	int retval = 0;
 	unsigned long value = 0;
 #ifdef LIRC_ON_SA1100
 	unsigned int ivalue;
-#endif
 
-#ifdef LIRC_SIR_TEKRAM
-	if (cmd == LIRC_GET_FEATURES)
-		value = LIRC_CAN_REC_MODE2;
-	else if (cmd == LIRC_GET_SEND_MODE)
-		value = 0;
-	else if (cmd == LIRC_GET_REC_MODE)
-		value = LIRC_MODE_MODE2;
-#elif defined(LIRC_ON_SA1100)
 	if (cmd == LIRC_GET_FEATURES)
 		value = LIRC_CAN_SEND_PULSE |
 			LIRC_CAN_SET_SEND_DUTY_CYCLE |
@@ -441,47 +423,42 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 		break;
 #ifdef LIRC_ON_SA1100
 	case LIRC_SET_SEND_DUTY_CYCLE:
-		retval=get_user(ivalue,(unsigned int *) arg);
-		if(retval) return(retval);
-		if(ivalue<=0 || ivalue>100) return(-EINVAL);
+		retval = get_user(ivalue, (unsigned int *) arg);
+		if (retval)
+			return reetval;
+		if (ivalue <= 0 || ivalue > 100)
+			return -EINVAL;
 		/* (ivalue/100)*(1000000/freq) */
-		duty_cycle=ivalue;
-		pulse_width=(unsigned long) duty_cycle*10000/freq;
-		space_width=(unsigned long) 1000000L/freq-pulse_width;
-		if(pulse_width>=LIRC_ON_SA1100_TRANSMITTER_LATENCY)
-			pulse_width-=LIRC_ON_SA1100_TRANSMITTER_LATENCY;
-		if(space_width>=LIRC_ON_SA1100_TRANSMITTER_LATENCY)
-			space_width-=LIRC_ON_SA1100_TRANSMITTER_LATENCY;
+		duty_cycle = ivalue;
+		pulse_width = (unsigned long) duty_cycle*10000/freq;
+		space_width = (unsigned long) 1000000L/freq-pulse_width;
+		if (pulse_width >= LIRC_ON_SA1100_TRANSMITTER_LATENCY)
+			pulse_width -= LIRC_ON_SA1100_TRANSMITTER_LATENCY;
+		if (space_width >= LIRC_ON_SA1100_TRANSMITTER_LATENCY)
+			space_width -= LIRC_ON_SA1100_TRANSMITTER_LATENCY;
 		break;
 	case LIRC_SET_SEND_CARRIER:
-		retval=get_user(ivalue,(unsigned int *) arg);
-		if(retval) return(retval);
-		if(ivalue>500000 || ivalue<20000) return(-EINVAL);
-		freq=ivalue;
-		pulse_width=(unsigned long) duty_cycle*10000/freq;
-		space_width=(unsigned long) 1000000L/freq-pulse_width;
-		if(pulse_width>=LIRC_ON_SA1100_TRANSMITTER_LATENCY)
-			pulse_width-=LIRC_ON_SA1100_TRANSMITTER_LATENCY;
-		if(space_width>=LIRC_ON_SA1100_TRANSMITTER_LATENCY)
-			space_width-=LIRC_ON_SA1100_TRANSMITTER_LATENCY;
+		retval = get_user(ivalue, (unsigned int *) arg);
+		if (retval)
+			return retval;
+		if (ivalue > 500000 || ivalue < 20000)
+			return -EINVAL;
+		freq = ivalue;
+		pulse_width = (unsigned long) duty_cycle*10000/freq;
+		space_width = (unsigned long) 1000000L/freq-pulse_width;
+		if (pulse_width >= LIRC_ON_SA1100_TRANSMITTER_LATENCY)
+			pulse_width -= LIRC_ON_SA1100_TRANSMITTER_LATENCY;
+		if (space_width >= LIRC_ON_SA1100_TRANSMITTER_LATENCY)
+			space_width -= LIRC_ON_SA1100_TRANSMITTER_LATENCY;
 		break;
 #endif
 	default:
 		retval = -ENOIOCTLCMD;
 
 	}
-	
+
 	if (retval)
 		return retval;
-	
-#ifdef LIRC_SIR_TEKRAM
-	if (cmd == LIRC_SET_REC_MODE) {
-		if (value != LIRC_MODE_MODE2)
-			retval = -ENOSYS;
-	} else if (cmd == LIRC_SET_SEND_MODE) {
-		retval = -ENOSYS;
-	}
-#else
 	if (cmd == LIRC_SET_REC_MODE) {
 		if (value != LIRC_MODE_MODE2)
 			retval = -ENOSYS;
@@ -489,7 +466,7 @@ static int lirc_ioctl(struct inode *node,struct file *filep,unsigned int cmd,
 		if (value != LIRC_MODE_PULSE)
 			retval = -ENOSYS;
 	}
-#endif
+
 	return retval;
 }
 
@@ -502,23 +479,19 @@ static void add_read_queue(int flag, unsigned long val)
 
 	newval = val & PULSE_MASK;
 
-	/* statistically pulses are ~TIME_CONST/2 too long: we could
-	   maybe make this more exactly but this is good enough */
-	if(flag) /* pulse */
-	{
-		if(newval>TIME_CONST/2)
-		{
-			newval-=TIME_CONST/2;
-		}
+	/*
+	 * statistically, pulses are ~TIME_CONST/2 too long. we could
+	 * maybe make this more exact, but this is good enough
+	 */
+	if (flag) {
+		/* pulse */
+		if (newval > TIME_CONST/2)
+			newval -= TIME_CONST/2;
 		else /* should not ever happen */
-		{
-			newval=1;
-		}
-		newval|=PULSE_BIT;
-	}
-	else
-	{
-		newval+=TIME_CONST/2;
+			newval = 1;
+		newval |= PULSE_BIT;
+	} else {
+		newval += TIME_CONST/2;
 	}
 	new_rx_tail = (rx_tail + 1) & (RBUF_LEN - 1);
 	if (new_rx_tail == rx_head) {
@@ -530,50 +503,48 @@ static void add_read_queue(int flag, unsigned long val)
 	wake_up_interruptible(&lirc_read_queue);
 }
 
-static struct file_operations lirc_fops =
-{
-	read:    lirc_read,
-	write:   lirc_write,
-	poll:    lirc_poll,
-	ioctl:   lirc_ioctl,
-	open:    lirc_open,
-	release: lirc_close,
+static struct file_operations lirc_fops = {
+	.owner		= THIS_MODULE,
+	.read		= lirc_read,
+	.write		= lirc_write,
+	.poll		= lirc_poll,
+	.ioctl		= lirc_ioctl,
+	.open		= lirc_open,
+	.release	= lirc_close,
 };
 
-static int set_use_inc(void* data)
+static int set_use_inc(void *data)
 {
-#if WE_DONT_USE_LOCAL_OPEN_CLOSE
-	MOD_INC_USE_COUNT;
-#endif
        return 0;
 }
 
-static void set_use_dec(void* data)
+static void set_use_dec(void *data)
 {
-#if WE_DONT_USE_LOCAL_OPEN_CLOSE
-	MOD_DEC_USE_COUNT;
-#endif
 }
-static struct lirc_plugin plugin = {
-       name:           LIRC_DRIVER_NAME,
-       minor:          -1,
-       code_length:    1,
-       sample_rate:    0,
-       data:           NULL,
-       add_to_buf:     NULL,
-       get_queue:      NULL,
-       set_use_inc:    set_use_inc,
-       set_use_dec:    set_use_dec,
-       fops:           &lirc_fops,
-       owner:          THIS_MODULE,
+
+static struct lirc_driver driver = {
+       .name		= LIRC_DRIVER_NAME,
+       .minor		= -1,
+       .code_length	= 1,
+       .sample_rate	= 0,
+       .data		= NULL,
+       .add_to_buf	= NULL,
+#ifndef LIRC_REMOVE_DURING_EXPORT
+       .get_queue	= NULL,
+#endif
+       .set_use_inc	= set_use_inc,
+       .set_use_dec	= set_use_dec,
+       .fops		= &lirc_fops,
+       .dev		= NULL,
+       .owner		= THIS_MODULE,
 };
 
 
 #ifdef MODULE
 static int init_chrdev(void)
 {
-	plugin.minor = lirc_register_plugin(&plugin);
-	if (plugin.minor < 0) {
+	driver.minor = lirc_register_driver(&driver);
+	if (driver.minor < 0) {
 		printk(KERN_ERR LIRC_DRIVER_NAME ": init_chrdev() failed.\n");
 		return -EIO;
 	}
@@ -582,15 +553,15 @@ static int init_chrdev(void)
 
 static void drop_chrdev(void)
 {
-	lirc_unregister_plugin(plugin.minor);
+	lirc_unregister_driver(driver.minor);
 }
 #endif
 
 /* SECTION: Hardware */
-static long delta(struct timeval * tv1, struct timeval * tv2)
+static long delta(struct timeval *tv1, struct timeval *tv2)
 {
 	unsigned long deltv;
-	
+
 	deltv = tv2->tv_sec - tv1->tv_sec;
 	if (deltv > 15)
 		deltv = 0xFFFFFF;
@@ -601,58 +572,61 @@ static long delta(struct timeval * tv1, struct timeval * tv2)
 	return deltv;
 }
 
-static void sir_timeout(unsigned long data) 
+static void sir_timeout(unsigned long data)
 {
-	/* if last received signal was a pulse, but receiving stopped
-	   within the 9 bit frame, we need to finish this pulse and
-	   simulate a signal change to from pulse to space. Otherwise
-	   upper layers will receive two sequences next time. */
-	
+	/*
+	 * if last received signal was a pulse, but receiving stopped
+	 * within the 9 bit frame, we need to finish this pulse and
+	 * simulate a signal change to from pulse to space. Otherwise
+	 * upper layers will receive two sequences next time.
+	 */
+
 	unsigned long flags;
 	unsigned long pulse_end;
-	
+
 	/* avoid interference with interrupt */
- 	spin_lock_irqsave(&timer_lock, flags);
-	if (last_value)
-	{
+	spin_lock_irqsave(&timer_lock, flags);
+	if (last_value) {
 #ifndef LIRC_ON_SA1100
 		/* clear unread bits in UART and restart */
 		outb(UART_FCR_CLEAR_RCVR, io + UART_FCR);
 #endif
 		/* determine 'virtual' pulse end: */
-	 	pulse_end = delta(&last_tv, &last_intr_tv);
-		dprintk("timeout add %d for %lu usec\n",last_value,pulse_end);
-		add_read_queue(last_value,pulse_end);
+		pulse_end = delta(&last_tv, &last_intr_tv);
+		dprintk("timeout add %d for %lu usec\n", last_value, pulse_end);
+		add_read_queue(last_value, pulse_end);
 		last_value = 0;
-		last_tv=last_intr_tv;
+		last_tv = last_intr_tv;
 	}
-	spin_unlock_irqrestore(&timer_lock, flags);		
+	spin_unlock_irqrestore(&timer_lock, flags);
 }
 
-static irqreturn_t sir_interrupt(int irq, void * dev_id,
-				 struct pt_regs * regs)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
+static irqreturn_t sir_interrupt(int irq, void *dev_id,
+				 struct pt_regs *regs)
+#else
+static irqreturn_t sir_interrupt(int irq, void *dev_id)
+#endif
 {
 	unsigned char data;
 	struct timeval curr_tv;
 	static unsigned long deltv;
 #ifdef LIRC_ON_SA1100
 	int status;
-	static int n=0;
-	
-	//printk("interrupt\n");
+	static int n;
+
 	status = Ser2UTSR0;
 	/*
 	 * Deal with any receive errors first.  The bytes in error may be
 	 * the only bytes in the receive FIFO, so we do this first.
 	 */
-	while (status & UTSR0_EIF)
-	{
+	while (status & UTSR0_EIF) {
 		int bstat;
 
-		if(debug) {
+		if (debug) {
 			dprintk("EIF\n");
 			bstat = Ser2UTSR1;
-		
+
 			if (bstat & UTSR1_FRE)
 				dprintk("frame error\n");
 			if (bstat & UTSR1_ROR)
@@ -660,45 +634,35 @@ static irqreturn_t sir_interrupt(int irq, void * dev_id,
 			if (bstat & UTSR1_PRE)
 				dprintk("parity error\n");
 		}
-		
+
 		bstat = Ser2UTDR;
 		n++;
 		status = Ser2UTSR0;
 	}
 
-	if (status & (UTSR0_RFS | UTSR0_RID))
-	{
+	if (status & (UTSR0_RFS | UTSR0_RID)) {
 		do_gettimeofday(&curr_tv);
 		deltv = delta(&last_tv, &curr_tv);
-		do
-		{
-			data=Ser2UTDR;
+		do {
+			data = Ser2UTDR;
 			dprintk("%d data: %u\n", n, (unsigned int) data);
 			n++;
-		}
-		while(status&UTSR0_RID && /* do not empty fifo in
-                                             order to get UTSR0_RID in
-                                             any case */
+		} while (status & UTSR0_RID && /* do not empty fifo in order to
+						* get UTSR0_RID in any case */
 		      Ser2UTSR1 & UTSR1_RNE); /* data ready */
-		
-		if(status&UTSR0_RID)
-		{
-			//printk("add\n");
-			add_read_queue(0,deltv-n*TIME_CONST); /*space*/
-			add_read_queue(1,n*TIME_CONST); /*pulse*/
-			n=0;
-			last_tv=curr_tv;
+
+		if (status&UTSR0_RID) {
+			add_read_queue(0 , deltv - n * TIME_CONST); /*space*/
+			add_read_queue(1, n * TIME_CONST); /*pulse*/
+			n = 0;
+			last_tv = curr_tv;
 		}
 	}
 
-	if (status & UTSR0_TFS) {
+	if (status & UTSR0_TFS)
+		printk(KERN_ERR "transmit fifo not full, shouldn't happen\n");
 
-		printk("transmit fifo not full, shouldn't ever happen\n");
-	}
-
-	/*
-	 * We must clear certain bits.
-	 */
+	/* We must clear certain bits. */
 	status &= (UTSR0_RID | UTSR0_RBB | UTSR0_REB);
 	if (status)
 		Ser2UTSR0 = status;
@@ -723,60 +687,64 @@ static irqreturn_t sir_interrupt(int irq, void * dev_id,
 			break;
 		case UART_IIR_RDI:
 			/* avoid interference with timer */
-		 	spin_lock_irqsave(&timer_lock, flags);
-			do
-			{
+			spin_lock_irqsave(&timer_lock, flags);
+			do {
 				del_timer(&timerlist);
 				data = inb(io + UART_RX);
 				do_gettimeofday(&curr_tv);
 				deltv = delta(&last_tv, &curr_tv);
 				deltintrtv = delta(&last_intr_tv, &curr_tv);
-				dprintk("t %lu , d %d\n", deltintrtv, (int)data);
-				/* if nothing came in last X cycles,
-				   it was gap */
+				dprintk("t %lu, d %d\n", deltintrtv, (int)data);
+				/*
+				 * if nothing came in last X cycles,
+				 * it was gap
+				 */
 				if (deltintrtv > TIME_CONST * threshold) {
 					if (last_value) {
 						dprintk("GAP\n");
 						/* simulate signal change */
 						add_read_queue(last_value,
-							       deltv-
+							       deltv -
 							       deltintrtv);
 						last_value = 0;
-						last_tv.tv_sec = last_intr_tv.tv_sec;
-						last_tv.tv_usec = last_intr_tv.tv_usec;
+						last_tv.tv_sec =
+							last_intr_tv.tv_sec;
+						last_tv.tv_usec =
+							last_intr_tv.tv_usec;
 						deltv = deltintrtv;
 					}
 				}
 				data = 1;
 				if (data ^ last_value) {
-					/* deltintrtv > 2*TIME_CONST,
-                                           remember ? */
-					/* the other case is timeout */
+					/*
+					 * deltintrtv > 2*TIME_CONST, remember?
+					 * the other case is timeout
+					 */
 					add_read_queue(last_value,
 						       deltv-TIME_CONST);
 					last_value = data;
 					last_tv = curr_tv;
-					if(last_tv.tv_usec>=TIME_CONST)
-					{
-						last_tv.tv_usec-=TIME_CONST;
-					}
-					else
-					{
+					if (last_tv.tv_usec >= TIME_CONST) {
+						last_tv.tv_usec -= TIME_CONST;
+					} else {
 						last_tv.tv_sec--;
-						last_tv.tv_usec+=1000000-
+						last_tv.tv_usec += 1000000 -
 							TIME_CONST;
 					}
 				}
 				last_intr_tv = curr_tv;
-				if (data)
-				{
-					/* start timer for end of sequence detection */
-					timerlist.expires = jiffies + SIR_TIMEOUT;
+				if (data) {
+					/*
+					 * start timer for end of
+					 * sequence detection
+					 */
+					timerlist.expires = jiffies +
+								SIR_TIMEOUT;
 					add_timer(&timerlist);
 				}
-			}
-			while ((lsr = inb(io + UART_LSR))
-				& UART_LSR_DR); /* data ready */
+
+				lsr = inb(io + UART_LSR);
+			} while (lsr & UART_LSR_DR); /* data ready */
 			spin_unlock_irqrestore(&timer_lock, flags);
 			break;
 		default:
@@ -790,24 +758,23 @@ static irqreturn_t sir_interrupt(int irq, void * dev_id,
 #ifdef LIRC_ON_SA1100
 static void send_pulse(unsigned long length)
 {
-	unsigned long k,delay;
+	unsigned long k, delay;
 	int flag;
 
-	if(length==0) return;
-	/* this won't give us the carrier frequency we really want
-	   due to integer arithmetic, but we can accept this inaccuracy */
+	if (length == 0)
+		return;
+	/*
+	 * this won't give us the carrier frequency we really want
+	 * due to integer arithmetic, but we can accept this inaccuracy
+	 */
 
-	for(k=flag=0;k<length;k+=delay,flag=!flag)
-	{
-		if(flag)
-		{
+	for (k = flag = 0; k < length; k += delay, flag = !flag) {
+		if (flag) {
 			off();
-			delay=space_width;
-		}
-		else
-		{
+			delay = space_width;
+		} else {
 			on();
-			delay=pulse_width;
+			delay = pulse_width;
 		}
 		safe_udelay(delay);
 	}
@@ -816,11 +783,11 @@ static void send_pulse(unsigned long length)
 
 static void send_space(unsigned long length)
 {
-	if(length==0) return;
+	if (length == 0)
+		return;
 	off();
 	safe_udelay(length);
 }
-#elif defined(LIRC_SIR_TEKRAM)
 #else
 static void send_space(unsigned long len)
 {
@@ -832,13 +799,16 @@ static void send_pulse(unsigned long len)
 	long bytes_out = len / TIME_CONST;
 	long time_left;
 
-	if (!bytes_out)
-		bytes_out++;
 	time_left = (long)len - (long)bytes_out * (long)TIME_CONST;
-	while (--bytes_out) {
+	if (bytes_out == 0) {
+		bytes_out++;
+		time_left = 0;
+	}
+	while (bytes_out--) {
 		outb(PULSE, io + UART_TX);
 		/* FIXME treba seriozne cakanie z drivers/char/serial.c */
-		while (!(inb(io + UART_LSR) & UART_LSR_THRE));
+		while (!(inb(io + UART_LSR) & UART_LSR_THRE))
+			;
 	}
 #if 0
 	if (time_left > 0)
@@ -848,7 +818,7 @@ static void send_pulse(unsigned long len)
 #endif
 
 #ifdef CONFIG_SA1100_COLLIE
-static inline int sa1100_irda_set_power_collie(int state)
+static int sa1100_irda_set_power_collie(int state)
 {
 	if (state) {
 		/*
@@ -861,8 +831,7 @@ static inline int sa1100_irda_set_power_collie(int state)
 					 TC35143_IODIR_OUTPUT);
 		ucb1200_set_io(TC35143_GPIO_IR_ON, TC35143_IODAT_LOW);
 		udelay(100);
-	}
-	else {
+	} else {
 		/* OFF */
 		ucb1200_set_io_direction(TC35143_GPIO_IR_ON,
 					 TC35143_IODIR_OUTPUT);
@@ -875,121 +844,119 @@ static inline int sa1100_irda_set_power_collie(int state)
 static int init_hardware(void)
 {
 	unsigned long flags;
-	
+
 	spin_lock_irqsave(&hardware_lock, flags);
 	/* reset UART */
 #ifdef LIRC_ON_SA1100
 #ifdef CONFIG_SA1100_BITSY
 	if (machine_is_bitsy()) {
-		printk("Power on IR module\n");
+		printk(KERN_INFO "Power on IR module\n");
 		set_bitsy_egpio(EGPIO_BITSY_IR_ON);
 	}
 #endif
 #ifdef CONFIG_SA1100_COLLIE
 	sa1100_irda_set_power_collie(3);	/* power on */
 #endif
-	sr.hscr0=Ser2HSCR0;
+	sr.hscr0 = Ser2HSCR0;
 
-	sr.utcr0=Ser2UTCR0;
-	sr.utcr1=Ser2UTCR1;
-	sr.utcr2=Ser2UTCR2;
-	sr.utcr3=Ser2UTCR3;
-	sr.utcr4=Ser2UTCR4;
+	sr.utcr0 = Ser2UTCR0;
+	sr.utcr1 = Ser2UTCR1;
+	sr.utcr2 = Ser2UTCR2;
+	sr.utcr3 = Ser2UTCR3;
+	sr.utcr4 = Ser2UTCR4;
 
-	sr.utdr=Ser2UTDR;
-	sr.utsr0=Ser2UTSR0;
-	sr.utsr1=Ser2UTSR1;
+	sr.utdr = Ser2UTDR;
+	sr.utsr0 = Ser2UTSR0;
+	sr.utsr1 = Ser2UTSR1;
 
 	/* configure GPIO */
 	/* output */
-	PPDR|=PPC_TXD2;
-	PSDR|=PPC_TXD2;
+	PPDR |= PPC_TXD2;
+	PSDR |= PPC_TXD2;
 	/* set output to 0 */
 	off();
-	
-	/*
-	 * Enable HP-SIR modulation, and ensure that the port is disabled.
-	 */
-	Ser2UTCR3=0;
-	Ser2HSCR0=sr.hscr0 & (~HSCR0_HSSP);
-	
+
+	/* Enable HP-SIR modulation, and ensure that the port is disabled. */
+	Ser2UTCR3 = 0;
+	Ser2HSCR0 = sr.hscr0 & (~HSCR0_HSSP);
+
 	/* clear status register to prevent unwanted interrupts */
 	Ser2UTSR0 &= (UTSR0_RID | UTSR0_RBB | UTSR0_REB);
-	
+
 	/* 7N1 */
-	Ser2UTCR0=UTCR0_1StpBit|UTCR0_7BitData;
+	Ser2UTCR0 = UTCR0_1StpBit|UTCR0_7BitData;
 	/* 115200 */
-	Ser2UTCR1=0;
-	Ser2UTCR2=1;
+	Ser2UTCR1 = 0;
+	Ser2UTCR2 = 1;
 	/* use HPSIR, 1.6 usec pulses */
-	Ser2UTCR4=UTCR4_HPSIR|UTCR4_Z1_6us;
-	
+	Ser2UTCR4 = UTCR4_HPSIR|UTCR4_Z1_6us;
+
 	/* enable receiver, receive fifo interrupt */
-	Ser2UTCR3=UTCR3_RXE|UTCR3_RIE;
-	
+	Ser2UTCR3 = UTCR3_RXE|UTCR3_RIE;
+
 	/* clear status register to prevent unwanted interrupts */
 	Ser2UTSR0 &= (UTSR0_RID | UTSR0_RBB | UTSR0_REB);
-	
+
 #elif defined(LIRC_SIR_TEKRAM)
-	/* disable FIFO */ 
+	/* disable FIFO */
 	soutp(UART_FCR,
 	      UART_FCR_CLEAR_RCVR|
 	      UART_FCR_CLEAR_XMIT|
 	      UART_FCR_TRIGGER_1);
-	
+
 	/* Set DLAB 0. */
 	soutp(UART_LCR, sinp(UART_LCR) & (~UART_LCR_DLAB));
-	
+
 	/* First of all, disable all interrupts */
-	soutp(UART_IER, sinp(UART_IER)&
+	soutp(UART_IER, sinp(UART_IER) &
 	      (~(UART_IER_MSI|UART_IER_RLSI|UART_IER_THRI|UART_IER_RDI)));
-	
+
 	/* Set DLAB 1. */
 	soutp(UART_LCR, sinp(UART_LCR) | UART_LCR_DLAB);
-	
+
 	/* Set divisor to 12 => 9600 Baud */
-	soutp(UART_DLM,0);
-	soutp(UART_DLL,12);
-	
+	soutp(UART_DLM, 0);
+	soutp(UART_DLL, 12);
+
 	/* Set DLAB 0. */
 	soutp(UART_LCR, sinp(UART_LCR) & (~UART_LCR_DLAB));
-	
+
 	/* power supply */
 	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
 	safe_udelay(50*1000);
-	
+
 	/* -DTR low -> reset PIC */
 	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_OUT2);
 	udelay(1*1000);
-	
+
 	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
 	udelay(100);
 
 
-        /* -RTS low -> send control byte */
+	/* -RTS low -> send control byte */
 	soutp(UART_MCR, UART_MCR_DTR|UART_MCR_OUT2);
 	udelay(7);
 	soutp(UART_TX, TEKRAM_115200|TEKRAM_PW);
-	
+
 	/* one byte takes ~1042 usec to transmit at 9600,8N1 */
 	udelay(1500);
-	
+
 	/* back to normal operation */
 	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
 	udelay(50);
 
 	udelay(1500);
-	
+
 	/* read previous control byte */
 	printk(KERN_INFO LIRC_DRIVER_NAME
-	       ": 0x%02x\n",sinp(UART_RX));
-	
+	       ": 0x%02x\n", sinp(UART_RX));
+
 	/* Set DLAB 1. */
 	soutp(UART_LCR, sinp(UART_LCR) | UART_LCR_DLAB);
-	
+
 	/* Set divisor to 1 => 115200 Baud */
-	soutp(UART_DLM,0);
-	soutp(UART_DLL,1);
+	soutp(UART_DLM, 0);
+	soutp(UART_DLL, 1);
 
 	/* Set DLAB 0, 8 Bit */
 	soutp(UART_LCR, UART_LCR_WLEN8);
@@ -999,20 +966,22 @@ static int init_hardware(void)
 	outb(0, io + UART_MCR);
 	outb(0, io + UART_IER);
 	/* init UART */
-		/* set DLAB, speed = 115200 */
+	/* set DLAB, speed = 115200 */
 	outb(UART_LCR_DLAB | UART_LCR_WLEN7, io + UART_LCR);
 	outb(1, io + UART_DLL); outb(0, io + UART_DLM);
-		/* 7N1+start = 9 bits at 115200 ~ 3 bits at 44000 */
+	/* 7N1+start = 9 bits at 115200 ~ 3 bits at 44000 */
 	outb(UART_LCR_WLEN7, io + UART_LCR);
-		/* FIFO operation */
+	/* FIFO operation */
 	outb(UART_FCR_ENABLE_FIFO, io + UART_FCR);
-		/* interrupts */
-	// outb(UART_IER_RLSI|UART_IER_RDI|UART_IER_THRI, io + UART_IER);
-	outb(UART_IER_RDI, io + UART_IER);	
+	/* interrupts */
+	/* outb(UART_IER_RLSI|UART_IER_RDI|UART_IER_THRI, io + UART_IER); */
+	outb(UART_IER_RDI, io + UART_IER);
 	/* turn on UART */
 	outb(UART_MCR_DTR|UART_MCR_RTS|UART_MCR_OUT2, io + UART_MCR);
 #ifdef LIRC_SIR_ACTISYS_ACT200L
 	init_act200();
+#elif defined(LIRC_SIR_ACTISYS_ACT220L)
+	init_act220();
 #endif
 #endif
 	spin_unlock_irqrestore(&hardware_lock, flags);
@@ -1026,26 +995,25 @@ static void drop_hardware(void)
 	spin_lock_irqsave(&hardware_lock, flags);
 
 #ifdef LIRC_ON_SA1100
-	Ser2UTCR3=0;
-	
-	Ser2UTCR0=sr.utcr0;
-	Ser2UTCR1=sr.utcr1;
-	Ser2UTCR2=sr.utcr2;
-	Ser2UTCR4=sr.utcr4;
-	Ser2UTCR3=sr.utcr3;
-	
-	Ser2HSCR0=sr.hscr0;
+	Ser2UTCR3 = 0;
+
+	Ser2UTCR0 = sr.utcr0;
+	Ser2UTCR1 = sr.utcr1;
+	Ser2UTCR2 = sr.utcr2;
+	Ser2UTCR4 = sr.utcr4;
+	Ser2UTCR3 = sr.utcr3;
+
+	Ser2HSCR0 = sr.hscr0;
 #ifdef CONFIG_SA1100_BITSY
-	if (machine_is_bitsy()) {
+	if (machine_is_bitsy())
 		clr_bitsy_egpio(EGPIO_BITSY_IR_ON);
-	}
 #endif
 #ifdef CONFIG_SA1100_COLLIE
 	sa1100_irda_set_power_collie(0);	/* power off */
 #endif
 #else
 	/* turn off interrupts */
-	outb(0, io + UART_IER);	
+	outb(0, io + UART_IER);
 #endif
 	spin_unlock_irqrestore(&hardware_lock, flags);
 }
@@ -1055,19 +1023,21 @@ static void drop_hardware(void)
 static int init_port(void)
 {
 	int retval;
-	
+
 	/* get I/O port access and IRQ line */
 #ifndef LIRC_ON_SA1100
-	if(request_region(io, 8, LIRC_DRIVER_NAME) == NULL)
-	{
+	if (request_region(io, 8, LIRC_DRIVER_NAME) == NULL) {
 		printk(KERN_ERR LIRC_DRIVER_NAME
 		       ": i/o port 0x%.4x already in use.\n", io);
 		return -EBUSY;
 	}
 #endif
-	retval = request_irq(irq, sir_interrupt, SA_INTERRUPT,
+	retval = request_irq(irq, sir_interrupt, IRQF_DISABLED,
 			     LIRC_DRIVER_NAME, NULL);
 	if (retval < 0) {
+#               ifndef LIRC_ON_SA1100
+		release_region(io, 8);
+#               endif
 		printk(KERN_ERR LIRC_DRIVER_NAME
 			": IRQ %d already in use.\n",
 			irq);
@@ -1096,11 +1066,10 @@ static void drop_port(void)
 }
 
 #ifdef LIRC_SIR_ACTISYS_ACT200L
-/******************************************************/
 /* Crystal/Cirrus CS8130 IR transceiver, used in Actisys Act200L dongle */
 /* some code borrowed from Linux IRDA driver */
 
-/* Regsiter 0: Control register #1 */
+/* Register 0: Control register #1 */
 #define ACT200L_REG0    0x00
 #define ACT200L_TXEN    0x01 /* Enable transmitter */
 #define ACT200L_RXEN    0x02 /* Enable receiver */
@@ -1137,7 +1106,7 @@ static void drop_port(void)
 #define ACT200L_REG7    0x70
 #define ACT200L_ENPOS   0x04 /* Ignore the falling edge */
 
-/* Register 8,9: Baud Rate Dvider register #1,#2 */
+/* Register 8,9: Baud Rate Divider register #1,#2 */
 #define ACT200L_REG8    0x80
 #define ACT200L_REG9    0x90
 
@@ -1181,26 +1150,24 @@ static void init_act200(void)
 
 	/* Set DLAB 1. */
 	soutp(UART_LCR, UART_LCR_DLAB | UART_LCR_WLEN8);
-	
+
 	/* Set divisor to 12 => 9600 Baud */
-	soutp(UART_DLM,0);
-	soutp(UART_DLL,12);
-	
+	soutp(UART_DLM, 0);
+	soutp(UART_DLL, 12);
+
 	/* Set DLAB 0. */
 	soutp(UART_LCR, UART_LCR_WLEN8);
 	/* Set divisor to 12 => 9600 Baud */
 
 	/* power supply */
 	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
-	for (i=0; i<50; i++) {
+	for (i = 0; i < 50; i++)
 		safe_udelay(1000);
-	}
 
 		/* Reset the dongle : set RTS low for 25 ms */
 	soutp(UART_MCR, UART_MCR_DTR|UART_MCR_OUT2);
-	for (i=0; i<25; i++) {
+	for (i = 0; i < 25; i++)
 		udelay(1000);
-	}
 
 	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
 	udelay(100);
@@ -1209,8 +1176,8 @@ static void init_act200(void)
 	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_OUT2);
 	udelay(7);
 
-/* send out the control register settings for 115K 7N1 SIR operation */
-	for (i=0; i<sizeof(control); i++) {
+	/* send out the control register settings for 115K 7N1 SIR operation */
+	for (i = 0; i < sizeof(control); i++) {
 		soutp(UART_TX, control[i]);
 		/* one byte takes ~1042 usec to transmit at 9600,8N1 */
 		udelay(1500);
@@ -1227,8 +1194,8 @@ static void init_act200(void)
 	soutp(UART_LCR, UART_LCR_DLAB | UART_LCR_WLEN7);
 
 	/* Set divisor to 1 => 115200 Baud */
-	soutp(UART_DLM,0);
-	soutp(UART_DLL,1);
+	soutp(UART_DLM, 0);
+	soutp(UART_DLL, 1);
 
 	/* Set DLAB 0. */
 	soutp(UART_LCR, sinp(UART_LCR) & (~UART_LCR_DLAB));
@@ -1241,6 +1208,65 @@ static void init_act200(void)
 }
 #endif
 
+#ifdef LIRC_SIR_ACTISYS_ACT220L
+/*
+ * Derived from linux IrDA driver (net/irda/actisys.c)
+ * Drop me a mail for any kind of comment: maxx@spaceboyz.net
+ */
+
+void init_act220(void)
+{
+	int i;
+
+	/* DLAB 1 */
+	soutp(UART_LCR, UART_LCR_DLAB|UART_LCR_WLEN7);
+
+	/* 9600 baud */
+	soutp(UART_DLM, 0);
+	soutp(UART_DLL, 12);
+
+	/* DLAB 0 */
+	soutp(UART_LCR, UART_LCR_WLEN7);
+
+	/* reset the dongle, set DTR low for 10us */
+	soutp(UART_MCR, UART_MCR_RTS|UART_MCR_OUT2);
+	udelay(10);
+
+	/* back to normal (still 9600) */
+	soutp(UART_MCR, UART_MCR_DTR|UART_MCR_RTS|UART_MCR_OUT2);
+
+	/*
+	 * send RTS pulses until we reach 115200
+	 * i hope this is really the same for act220l/act220l+
+	 */
+	for (i = 0; i < 3; i++) {
+		udelay(10);
+		/* set RTS low for 10 us */
+		soutp(UART_MCR, UART_MCR_DTR|UART_MCR_OUT2);
+		udelay(10);
+		/* set RTS high for 10 us */
+		soutp(UART_MCR, UART_MCR_RTS|UART_MCR_DTR|UART_MCR_OUT2);
+	}
+
+	/* back to normal operation */
+	udelay(1500); /* better safe than sorry ;) */
+
+	/* Set DLAB 1. */
+	soutp(UART_LCR, UART_LCR_DLAB | UART_LCR_WLEN7);
+
+	/* Set divisor to 1 => 115200 Baud */
+	soutp(UART_DLM, 0);
+	soutp(UART_DLL, 1);
+
+	/* Set DLAB 0, 7 Bit */
+	/* The dongle doesn't seem to have any problems with operation at 7N1 */
+	soutp(UART_LCR, UART_LCR_WLEN7);
+
+	/* enable interrupts */
+	soutp(UART_IER, UART_IER_RDI);
+}
+#endif
+
 static int init_lirc_sir(void)
 {
 	int retval;
@@ -1257,12 +1283,12 @@ static int init_lirc_sir(void)
 
 #ifdef MODULE
 
-int init_module(void)
+static int __init lirc_sir_init(void)
 {
 	int retval;
-	
-	retval=init_chrdev();
-	if(retval < 0)
+
+	retval = init_chrdev();
+	if (retval < 0)
 		return retval;
 	retval = init_lirc_sir();
 	if (retval) {
@@ -1272,7 +1298,7 @@ int init_module(void)
 	return 0;
 }
 
-void cleanup_module(void)
+static void __exit lirc_sir_exit(void)
 {
 	drop_hardware();
 	drop_chrdev();
@@ -1280,6 +1306,9 @@ void cleanup_module(void)
 	printk(KERN_INFO LIRC_DRIVER_NAME ": Uninstalled.\n");
 }
 
+module_init(lirc_sir_init);
+module_exit(lirc_sir_exit);
+
 #ifdef LIRC_SIR_TEKRAM
 MODULE_DESCRIPTION("Infrared receiver driver for Tekram Irmate 210");
 MODULE_AUTHOR("Christoph Bartelmus");
@@ -1289,6 +1318,9 @@ MODULE_AUTHOR("Christoph Bartelmus");
 #elif defined(LIRC_SIR_ACTISYS_ACT200L)
 MODULE_DESCRIPTION("LIRC driver for Actisys Act200L");
 MODULE_AUTHOR("Karl Bongers");
+#elif defined(LIRC_SIR_ACTISYS_ACT220L)
+MODULE_DESCRIPTION("LIRC driver for Actisys Act220L(+)");
+MODULE_AUTHOR("Jan Roemisch");
 #else
 MODULE_DESCRIPTION("Infrared receiver driver for SIR type serial ports");
 MODULE_AUTHOR("Milan Pikula");
@@ -1296,20 +1328,20 @@ MODULE_AUTHOR("Milan Pikula");
 MODULE_LICENSE("GPL");
 
 #ifdef LIRC_ON_SA1100
-module_param(irq, int, 0444);
+module_param(irq, int, S_IRUGO);
 MODULE_PARM_DESC(irq, "Interrupt (16)");
 #else
-module_param(io, int, 0444);
+module_param(io, int, S_IRUGO);
 MODULE_PARM_DESC(io, "I/O address base (0x3f8 or 0x2f8)");
 
-module_param(irq, int, 0444);
+module_param(irq, int, S_IRUGO);
 MODULE_PARM_DESC(irq, "Interrupt (4 or 3)");
 
-module_param(threshold, int, 0444);
+module_param(threshold, int, S_IRUGO);
 MODULE_PARM_DESC(threshold, "space detection threshold (3)");
 #endif
 
-module_param(debug, bool, 0644);
+module_param(debug, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Enable debugging messages");
 
 EXPORT_NO_SYMBOLS;
